from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricAdmittanceUnits(Enum):
        """
            ElectricAdmittanceUnits enumeration
        """
        
        Siemens = 'Siemens'
        """
            
        """
        
        Nanosiemens = 'Nanosiemens'
        """
            
        """
        
        Microsiemens = 'Microsiemens'
        """
            
        """
        
        Millisiemens = 'Millisiemens'
        """
            
        """
        

class ElectricAdmittanceDto:
    """
    A DTO representation of a ElectricAdmittance

    Attributes:
        value (float): The value of the ElectricAdmittance.
        unit (ElectricAdmittanceUnits): The specific unit that the ElectricAdmittance value is representing.
    """

    def __init__(self, value: float, unit: ElectricAdmittanceUnits):
        """
        Create a new DTO representation of a ElectricAdmittance

        Parameters:
            value (float): The value of the ElectricAdmittance.
            unit (ElectricAdmittanceUnits): The specific unit that the ElectricAdmittance value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricAdmittance
        """
        self.unit: ElectricAdmittanceUnits = unit
        """
        The specific unit that the ElectricAdmittance value is representing
        """

    def to_json(self):
        """
        Get a ElectricAdmittance DTO JSON object representing the current unit.

        :return: JSON object represents ElectricAdmittance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Siemens"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricAdmittance DTO from a json representation.

        :param data: The ElectricAdmittance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Siemens"}
        :return: A new instance of ElectricAdmittanceDto.
        :rtype: ElectricAdmittanceDto
        """
        return ElectricAdmittanceDto(value=data["value"], unit=ElectricAdmittanceUnits(data["unit"]))


class ElectricAdmittance(AbstractMeasure):
    """
    Electric admittance is a measure of how easily a circuit or device will allow a current to flow. It is defined as the inverse of impedance. The SI unit of admittance is the siemens (symbol S).

    Args:
        value (float): The value.
        from_unit (ElectricAdmittanceUnits): The ElectricAdmittance unit to create from, The default unit is Siemens
    """
    def __init__(self, value: float, from_unit: ElectricAdmittanceUnits = ElectricAdmittanceUnits.Siemens):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__siemens = None
        
        self.__nanosiemens = None
        
        self.__microsiemens = None
        
        self.__millisiemens = None
        

    def convert(self, unit: ElectricAdmittanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricAdmittanceUnits = ElectricAdmittanceUnits.Siemens) -> ElectricAdmittanceDto:
        """
        Get a new instance of ElectricAdmittance DTO representing the current unit.

        :param hold_in_unit: The specific ElectricAdmittance unit to store the ElectricAdmittance value in the DTO representation.
        :type hold_in_unit: ElectricAdmittanceUnits
        :return: A new instance of ElectricAdmittanceDto.
        :rtype: ElectricAdmittanceDto
        """
        return ElectricAdmittanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricAdmittanceUnits = ElectricAdmittanceUnits.Siemens):
        """
        Get a ElectricAdmittance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricAdmittance unit to store the ElectricAdmittance value in the DTO representation.
        :type hold_in_unit: ElectricAdmittanceUnits
        :return: JSON object represents ElectricAdmittance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Siemens"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_admittance_dto: ElectricAdmittanceDto):
        """
        Obtain a new instance of ElectricAdmittance from a DTO unit object.

        :param electric_admittance_dto: The ElectricAdmittance DTO representation.
        :type electric_admittance_dto: ElectricAdmittanceDto
        :return: A new instance of ElectricAdmittance.
        :rtype: ElectricAdmittance
        """
        return ElectricAdmittance(electric_admittance_dto.value, electric_admittance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricAdmittance from a DTO unit json representation.

        :param data: The ElectricAdmittance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Siemens"}
        :return: A new instance of ElectricAdmittance.
        :rtype: ElectricAdmittance
        """
        return ElectricAdmittance.from_dto(ElectricAdmittanceDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricAdmittanceUnits) -> float:
        value = self._value
        
        if from_unit == ElectricAdmittanceUnits.Siemens:
            return (value)
        
        if from_unit == ElectricAdmittanceUnits.Nanosiemens:
            return ((value) / 1e-09)
        
        if from_unit == ElectricAdmittanceUnits.Microsiemens:
            return ((value) / 1e-06)
        
        if from_unit == ElectricAdmittanceUnits.Millisiemens:
            return ((value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricAdmittanceUnits) -> float:
        
        if to_unit == ElectricAdmittanceUnits.Siemens:
            return (value)
        
        if to_unit == ElectricAdmittanceUnits.Nanosiemens:
            return ((value) * 1e-09)
        
        if to_unit == ElectricAdmittanceUnits.Microsiemens:
            return ((value) * 1e-06)
        
        if to_unit == ElectricAdmittanceUnits.Millisiemens:
            return ((value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_siemens(siemens: float):
        """
        Create a new instance of ElectricAdmittance from a value in siemens.

        

        :param meters: The ElectricAdmittance value in siemens.
        :type siemens: float
        :return: A new instance of ElectricAdmittance.
        :rtype: ElectricAdmittance
        """
        return ElectricAdmittance(siemens, ElectricAdmittanceUnits.Siemens)

    
    @staticmethod
    def from_nanosiemens(nanosiemens: float):
        """
        Create a new instance of ElectricAdmittance from a value in nanosiemens.

        

        :param meters: The ElectricAdmittance value in nanosiemens.
        :type nanosiemens: float
        :return: A new instance of ElectricAdmittance.
        :rtype: ElectricAdmittance
        """
        return ElectricAdmittance(nanosiemens, ElectricAdmittanceUnits.Nanosiemens)

    
    @staticmethod
    def from_microsiemens(microsiemens: float):
        """
        Create a new instance of ElectricAdmittance from a value in microsiemens.

        

        :param meters: The ElectricAdmittance value in microsiemens.
        :type microsiemens: float
        :return: A new instance of ElectricAdmittance.
        :rtype: ElectricAdmittance
        """
        return ElectricAdmittance(microsiemens, ElectricAdmittanceUnits.Microsiemens)

    
    @staticmethod
    def from_millisiemens(millisiemens: float):
        """
        Create a new instance of ElectricAdmittance from a value in millisiemens.

        

        :param meters: The ElectricAdmittance value in millisiemens.
        :type millisiemens: float
        :return: A new instance of ElectricAdmittance.
        :rtype: ElectricAdmittance
        """
        return ElectricAdmittance(millisiemens, ElectricAdmittanceUnits.Millisiemens)

    
    @property
    def siemens(self) -> float:
        """
        
        """
        if self.__siemens != None:
            return self.__siemens
        self.__siemens = self.__convert_from_base(ElectricAdmittanceUnits.Siemens)
        return self.__siemens

    
    @property
    def nanosiemens(self) -> float:
        """
        
        """
        if self.__nanosiemens != None:
            return self.__nanosiemens
        self.__nanosiemens = self.__convert_from_base(ElectricAdmittanceUnits.Nanosiemens)
        return self.__nanosiemens

    
    @property
    def microsiemens(self) -> float:
        """
        
        """
        if self.__microsiemens != None:
            return self.__microsiemens
        self.__microsiemens = self.__convert_from_base(ElectricAdmittanceUnits.Microsiemens)
        return self.__microsiemens

    
    @property
    def millisiemens(self) -> float:
        """
        
        """
        if self.__millisiemens != None:
            return self.__millisiemens
        self.__millisiemens = self.__convert_from_base(ElectricAdmittanceUnits.Millisiemens)
        return self.__millisiemens

    
    def to_string(self, unit: ElectricAdmittanceUnits = ElectricAdmittanceUnits.Siemens, fractional_digits: int = None) -> str:
        """
        Format the ElectricAdmittance to a string.
        
        Note: the default format for ElectricAdmittance is Siemens.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricAdmittance. Default is 'Siemens'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricAdmittanceUnits.Siemens:
            return f"""{super()._truncate_fraction_digits(self.siemens, fractional_digits)} S"""
        
        if unit == ElectricAdmittanceUnits.Nanosiemens:
            return f"""{super()._truncate_fraction_digits(self.nanosiemens, fractional_digits)} nS"""
        
        if unit == ElectricAdmittanceUnits.Microsiemens:
            return f"""{super()._truncate_fraction_digits(self.microsiemens, fractional_digits)} μS"""
        
        if unit == ElectricAdmittanceUnits.Millisiemens:
            return f"""{super()._truncate_fraction_digits(self.millisiemens, fractional_digits)} mS"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricAdmittanceUnits = ElectricAdmittanceUnits.Siemens) -> str:
        """
        Get ElectricAdmittance unit abbreviation.
        Note! the default abbreviation for ElectricAdmittance is Siemens.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricAdmittanceUnits.Siemens:
            return """S"""
        
        if unit_abbreviation == ElectricAdmittanceUnits.Nanosiemens:
            return """nS"""
        
        if unit_abbreviation == ElectricAdmittanceUnits.Microsiemens:
            return """μS"""
        
        if unit_abbreviation == ElectricAdmittanceUnits.Millisiemens:
            return """mS"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class EnergyUnits(Enum):
        """
            EnergyUnits enumeration
        """
        
        Joule = 'Joule'
        """
            
        """
        
        Calorie = 'Calorie'
        """
            
        """
        
        BritishThermalUnit = 'BritishThermalUnit'
        """
            
        """
        
        ElectronVolt = 'ElectronVolt'
        """
            
        """
        
        FootPound = 'FootPound'
        """
            
        """
        
        Erg = 'Erg'
        """
            
        """
        
        WattHour = 'WattHour'
        """
            
        """
        
        WattDay = 'WattDay'
        """
            
        """
        
        ThermEc = 'ThermEc'
        """
            
        """
        
        ThermUs = 'ThermUs'
        """
            
        """
        
        ThermImperial = 'ThermImperial'
        """
            
        """
        
        HorsepowerHour = 'HorsepowerHour'
        """
            
        """
        
        Nanojoule = 'Nanojoule'
        """
            
        """
        
        Microjoule = 'Microjoule'
        """
            
        """
        
        Millijoule = 'Millijoule'
        """
            
        """
        
        Kilojoule = 'Kilojoule'
        """
            
        """
        
        Megajoule = 'Megajoule'
        """
            
        """
        
        Gigajoule = 'Gigajoule'
        """
            
        """
        
        Terajoule = 'Terajoule'
        """
            
        """
        
        Petajoule = 'Petajoule'
        """
            
        """
        
        Kilocalorie = 'Kilocalorie'
        """
            
        """
        
        Megacalorie = 'Megacalorie'
        """
            
        """
        
        KilobritishThermalUnit = 'KilobritishThermalUnit'
        """
            
        """
        
        MegabritishThermalUnit = 'MegabritishThermalUnit'
        """
            
        """
        
        GigabritishThermalUnit = 'GigabritishThermalUnit'
        """
            
        """
        
        KiloelectronVolt = 'KiloelectronVolt'
        """
            
        """
        
        MegaelectronVolt = 'MegaelectronVolt'
        """
            
        """
        
        GigaelectronVolt = 'GigaelectronVolt'
        """
            
        """
        
        TeraelectronVolt = 'TeraelectronVolt'
        """
            
        """
        
        KilowattHour = 'KilowattHour'
        """
            
        """
        
        MegawattHour = 'MegawattHour'
        """
            
        """
        
        GigawattHour = 'GigawattHour'
        """
            
        """
        
        TerawattHour = 'TerawattHour'
        """
            
        """
        
        KilowattDay = 'KilowattDay'
        """
            
        """
        
        MegawattDay = 'MegawattDay'
        """
            
        """
        
        GigawattDay = 'GigawattDay'
        """
            
        """
        
        TerawattDay = 'TerawattDay'
        """
            
        """
        
        DecathermEc = 'DecathermEc'
        """
            
        """
        
        DecathermUs = 'DecathermUs'
        """
            
        """
        
        DecathermImperial = 'DecathermImperial'
        """
            
        """
        

class EnergyDto:
    """
    A DTO representation of a Energy

    Attributes:
        value (float): The value of the Energy.
        unit (EnergyUnits): The specific unit that the Energy value is representing.
    """

    def __init__(self, value: float, unit: EnergyUnits):
        """
        Create a new DTO representation of a Energy

        Parameters:
            value (float): The value of the Energy.
            unit (EnergyUnits): The specific unit that the Energy value is representing.
        """
        self.value: float = value
        """
        The value of the Energy
        """
        self.unit: EnergyUnits = unit
        """
        The specific unit that the Energy value is representing
        """

    def to_json(self):
        """
        Get a Energy DTO JSON object representing the current unit.

        :return: JSON object represents Energy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Joule"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Energy DTO from a json representation.

        :param data: The Energy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Joule"}
        :return: A new instance of EnergyDto.
        :rtype: EnergyDto
        """
        return EnergyDto(value=data["value"], unit=EnergyUnits(data["unit"]))


class Energy(AbstractMeasure):
    """
    The joule, symbol J, is a derived unit of energy, work, or amount of heat in the International System of Units. It is equal to the energy transferred (or work done) when applying a force of one newton through a distance of one metre (1 newton metre or N·m), or in passing an electric current of one ampere through a resistance of one ohm for one second. Many other units of energy are included. Please do not confuse this definition of the calorie with the one colloquially used by the food industry, the large calorie, which is equivalent to 1 kcal. Thermochemical definition of the calorie is used. For BTU, the IT definition is used.

    Args:
        value (float): The value.
        from_unit (EnergyUnits): The Energy unit to create from, The default unit is Joule
    """
    def __init__(self, value: float, from_unit: EnergyUnits = EnergyUnits.Joule):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules = None
        
        self.__calories = None
        
        self.__british_thermal_units = None
        
        self.__electron_volts = None
        
        self.__foot_pounds = None
        
        self.__ergs = None
        
        self.__watt_hours = None
        
        self.__watt_days = None
        
        self.__therms_ec = None
        
        self.__therms_us = None
        
        self.__therms_imperial = None
        
        self.__horsepower_hours = None
        
        self.__nanojoules = None
        
        self.__microjoules = None
        
        self.__millijoules = None
        
        self.__kilojoules = None
        
        self.__megajoules = None
        
        self.__gigajoules = None
        
        self.__terajoules = None
        
        self.__petajoules = None
        
        self.__kilocalories = None
        
        self.__megacalories = None
        
        self.__kilobritish_thermal_units = None
        
        self.__megabritish_thermal_units = None
        
        self.__gigabritish_thermal_units = None
        
        self.__kiloelectron_volts = None
        
        self.__megaelectron_volts = None
        
        self.__gigaelectron_volts = None
        
        self.__teraelectron_volts = None
        
        self.__kilowatt_hours = None
        
        self.__megawatt_hours = None
        
        self.__gigawatt_hours = None
        
        self.__terawatt_hours = None
        
        self.__kilowatt_days = None
        
        self.__megawatt_days = None
        
        self.__gigawatt_days = None
        
        self.__terawatt_days = None
        
        self.__decatherms_ec = None
        
        self.__decatherms_us = None
        
        self.__decatherms_imperial = None
        

    def convert(self, unit: EnergyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: EnergyUnits = EnergyUnits.Joule) -> EnergyDto:
        """
        Get a new instance of Energy DTO representing the current unit.

        :param hold_in_unit: The specific Energy unit to store the Energy value in the DTO representation.
        :type hold_in_unit: EnergyUnits
        :return: A new instance of EnergyDto.
        :rtype: EnergyDto
        """
        return EnergyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: EnergyUnits = EnergyUnits.Joule):
        """
        Get a Energy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Energy unit to store the Energy value in the DTO representation.
        :type hold_in_unit: EnergyUnits
        :return: JSON object represents Energy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Joule"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(energy_dto: EnergyDto):
        """
        Obtain a new instance of Energy from a DTO unit object.

        :param energy_dto: The Energy DTO representation.
        :type energy_dto: EnergyDto
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(energy_dto.value, energy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Energy from a DTO unit json representation.

        :param data: The Energy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Joule"}
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy.from_dto(EnergyDto.from_json(data))

    def __convert_from_base(self, from_unit: EnergyUnits) -> float:
        value = self._value
        
        if from_unit == EnergyUnits.Joule:
            return (value)
        
        if from_unit == EnergyUnits.Calorie:
            return (value / 4.184)
        
        if from_unit == EnergyUnits.BritishThermalUnit:
            return (value / 1055.05585262)
        
        if from_unit == EnergyUnits.ElectronVolt:
            return (value / 1.602176565e-19)
        
        if from_unit == EnergyUnits.FootPound:
            return (value / 1.355817948)
        
        if from_unit == EnergyUnits.Erg:
            return (value / 1e-7)
        
        if from_unit == EnergyUnits.WattHour:
            return (value / 3600)
        
        if from_unit == EnergyUnits.WattDay:
            return (value / (24 * 3600))
        
        if from_unit == EnergyUnits.ThermEc:
            return (value / 1.05505585262e8)
        
        if from_unit == EnergyUnits.ThermUs:
            return (value / 1.054804e8)
        
        if from_unit == EnergyUnits.ThermImperial:
            return (value / 1.05505585257348e8)
        
        if from_unit == EnergyUnits.HorsepowerHour:
            return (value / 2.6845195377e6)
        
        if from_unit == EnergyUnits.Nanojoule:
            return ((value) / 1e-09)
        
        if from_unit == EnergyUnits.Microjoule:
            return ((value) / 1e-06)
        
        if from_unit == EnergyUnits.Millijoule:
            return ((value) / 0.001)
        
        if from_unit == EnergyUnits.Kilojoule:
            return ((value) / 1000.0)
        
        if from_unit == EnergyUnits.Megajoule:
            return ((value) / 1000000.0)
        
        if from_unit == EnergyUnits.Gigajoule:
            return ((value) / 1000000000.0)
        
        if from_unit == EnergyUnits.Terajoule:
            return ((value) / 1000000000000.0)
        
        if from_unit == EnergyUnits.Petajoule:
            return ((value) / 1000000000000000.0)
        
        if from_unit == EnergyUnits.Kilocalorie:
            return ((value / 4.184) / 1000.0)
        
        if from_unit == EnergyUnits.Megacalorie:
            return ((value / 4.184) / 1000000.0)
        
        if from_unit == EnergyUnits.KilobritishThermalUnit:
            return ((value / 1055.05585262) / 1000.0)
        
        if from_unit == EnergyUnits.MegabritishThermalUnit:
            return ((value / 1055.05585262) / 1000000.0)
        
        if from_unit == EnergyUnits.GigabritishThermalUnit:
            return ((value / 1055.05585262) / 1000000000.0)
        
        if from_unit == EnergyUnits.KiloelectronVolt:
            return ((value / 1.602176565e-19) / 1000.0)
        
        if from_unit == EnergyUnits.MegaelectronVolt:
            return ((value / 1.602176565e-19) / 1000000.0)
        
        if from_unit == EnergyUnits.GigaelectronVolt:
            return ((value / 1.602176565e-19) / 1000000000.0)
        
        if from_unit == EnergyUnits.TeraelectronVolt:
            return ((value / 1.602176565e-19) / 1000000000000.0)
        
        if from_unit == EnergyUnits.KilowattHour:
            return ((value / 3600) / 1000.0)
        
        if from_unit == EnergyUnits.MegawattHour:
            return ((value / 3600) / 1000000.0)
        
        if from_unit == EnergyUnits.GigawattHour:
            return ((value / 3600) / 1000000000.0)
        
        if from_unit == EnergyUnits.TerawattHour:
            return ((value / 3600) / 1000000000000.0)
        
        if from_unit == EnergyUnits.KilowattDay:
            return ((value / (24 * 3600)) / 1000.0)
        
        if from_unit == EnergyUnits.MegawattDay:
            return ((value / (24 * 3600)) / 1000000.0)
        
        if from_unit == EnergyUnits.GigawattDay:
            return ((value / (24 * 3600)) / 1000000000.0)
        
        if from_unit == EnergyUnits.TerawattDay:
            return ((value / (24 * 3600)) / 1000000000000.0)
        
        if from_unit == EnergyUnits.DecathermEc:
            return ((value / 1.05505585262e8) / 10.0)
        
        if from_unit == EnergyUnits.DecathermUs:
            return ((value / 1.054804e8) / 10.0)
        
        if from_unit == EnergyUnits.DecathermImperial:
            return ((value / 1.05505585257348e8) / 10.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: EnergyUnits) -> float:
        
        if to_unit == EnergyUnits.Joule:
            return (value)
        
        if to_unit == EnergyUnits.Calorie:
            return (value * 4.184)
        
        if to_unit == EnergyUnits.BritishThermalUnit:
            return (value * 1055.05585262)
        
        if to_unit == EnergyUnits.ElectronVolt:
            return (value * 1.602176565e-19)
        
        if to_unit == EnergyUnits.FootPound:
            return (value * 1.355817948)
        
        if to_unit == EnergyUnits.Erg:
            return (value * 1e-7)
        
        if to_unit == EnergyUnits.WattHour:
            return (value * 3600)
        
        if to_unit == EnergyUnits.WattDay:
            return (value * 24 * 3600)
        
        if to_unit == EnergyUnits.ThermEc:
            return (value * 1.05505585262e8)
        
        if to_unit == EnergyUnits.ThermUs:
            return (value * 1.054804e8)
        
        if to_unit == EnergyUnits.ThermImperial:
            return (value * 1.05505585257348e8)
        
        if to_unit == EnergyUnits.HorsepowerHour:
            return (value * 2.6845195377e6)
        
        if to_unit == EnergyUnits.Nanojoule:
            return ((value) * 1e-09)
        
        if to_unit == EnergyUnits.Microjoule:
            return ((value) * 1e-06)
        
        if to_unit == EnergyUnits.Millijoule:
            return ((value) * 0.001)
        
        if to_unit == EnergyUnits.Kilojoule:
            return ((value) * 1000.0)
        
        if to_unit == EnergyUnits.Megajoule:
            return ((value) * 1000000.0)
        
        if to_unit == EnergyUnits.Gigajoule:
            return ((value) * 1000000000.0)
        
        if to_unit == EnergyUnits.Terajoule:
            return ((value) * 1000000000000.0)
        
        if to_unit == EnergyUnits.Petajoule:
            return ((value) * 1000000000000000.0)
        
        if to_unit == EnergyUnits.Kilocalorie:
            return ((value * 4.184) * 1000.0)
        
        if to_unit == EnergyUnits.Megacalorie:
            return ((value * 4.184) * 1000000.0)
        
        if to_unit == EnergyUnits.KilobritishThermalUnit:
            return ((value * 1055.05585262) * 1000.0)
        
        if to_unit == EnergyUnits.MegabritishThermalUnit:
            return ((value * 1055.05585262) * 1000000.0)
        
        if to_unit == EnergyUnits.GigabritishThermalUnit:
            return ((value * 1055.05585262) * 1000000000.0)
        
        if to_unit == EnergyUnits.KiloelectronVolt:
            return ((value * 1.602176565e-19) * 1000.0)
        
        if to_unit == EnergyUnits.MegaelectronVolt:
            return ((value * 1.602176565e-19) * 1000000.0)
        
        if to_unit == EnergyUnits.GigaelectronVolt:
            return ((value * 1.602176565e-19) * 1000000000.0)
        
        if to_unit == EnergyUnits.TeraelectronVolt:
            return ((value * 1.602176565e-19) * 1000000000000.0)
        
        if to_unit == EnergyUnits.KilowattHour:
            return ((value * 3600) * 1000.0)
        
        if to_unit == EnergyUnits.MegawattHour:
            return ((value * 3600) * 1000000.0)
        
        if to_unit == EnergyUnits.GigawattHour:
            return ((value * 3600) * 1000000000.0)
        
        if to_unit == EnergyUnits.TerawattHour:
            return ((value * 3600) * 1000000000000.0)
        
        if to_unit == EnergyUnits.KilowattDay:
            return ((value * 24 * 3600) * 1000.0)
        
        if to_unit == EnergyUnits.MegawattDay:
            return ((value * 24 * 3600) * 1000000.0)
        
        if to_unit == EnergyUnits.GigawattDay:
            return ((value * 24 * 3600) * 1000000000.0)
        
        if to_unit == EnergyUnits.TerawattDay:
            return ((value * 24 * 3600) * 1000000000000.0)
        
        if to_unit == EnergyUnits.DecathermEc:
            return ((value * 1.05505585262e8) * 10.0)
        
        if to_unit == EnergyUnits.DecathermUs:
            return ((value * 1.054804e8) * 10.0)
        
        if to_unit == EnergyUnits.DecathermImperial:
            return ((value * 1.05505585257348e8) * 10.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules(joules: float):
        """
        Create a new instance of Energy from a value in joules.

        

        :param meters: The Energy value in joules.
        :type joules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(joules, EnergyUnits.Joule)

    
    @staticmethod
    def from_calories(calories: float):
        """
        Create a new instance of Energy from a value in calories.

        

        :param meters: The Energy value in calories.
        :type calories: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(calories, EnergyUnits.Calorie)

    
    @staticmethod
    def from_british_thermal_units(british_thermal_units: float):
        """
        Create a new instance of Energy from a value in british_thermal_units.

        

        :param meters: The Energy value in british_thermal_units.
        :type british_thermal_units: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(british_thermal_units, EnergyUnits.BritishThermalUnit)

    
    @staticmethod
    def from_electron_volts(electron_volts: float):
        """
        Create a new instance of Energy from a value in electron_volts.

        

        :param meters: The Energy value in electron_volts.
        :type electron_volts: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(electron_volts, EnergyUnits.ElectronVolt)

    
    @staticmethod
    def from_foot_pounds(foot_pounds: float):
        """
        Create a new instance of Energy from a value in foot_pounds.

        

        :param meters: The Energy value in foot_pounds.
        :type foot_pounds: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(foot_pounds, EnergyUnits.FootPound)

    
    @staticmethod
    def from_ergs(ergs: float):
        """
        Create a new instance of Energy from a value in ergs.

        

        :param meters: The Energy value in ergs.
        :type ergs: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(ergs, EnergyUnits.Erg)

    
    @staticmethod
    def from_watt_hours(watt_hours: float):
        """
        Create a new instance of Energy from a value in watt_hours.

        

        :param meters: The Energy value in watt_hours.
        :type watt_hours: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(watt_hours, EnergyUnits.WattHour)

    
    @staticmethod
    def from_watt_days(watt_days: float):
        """
        Create a new instance of Energy from a value in watt_days.

        

        :param meters: The Energy value in watt_days.
        :type watt_days: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(watt_days, EnergyUnits.WattDay)

    
    @staticmethod
    def from_therms_ec(therms_ec: float):
        """
        Create a new instance of Energy from a value in therms_ec.

        

        :param meters: The Energy value in therms_ec.
        :type therms_ec: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(therms_ec, EnergyUnits.ThermEc)

    
    @staticmethod
    def from_therms_us(therms_us: float):
        """
        Create a new instance of Energy from a value in therms_us.

        

        :param meters: The Energy value in therms_us.
        :type therms_us: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(therms_us, EnergyUnits.ThermUs)

    
    @staticmethod
    def from_therms_imperial(therms_imperial: float):
        """
        Create a new instance of Energy from a value in therms_imperial.

        

        :param meters: The Energy value in therms_imperial.
        :type therms_imperial: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(therms_imperial, EnergyUnits.ThermImperial)

    
    @staticmethod
    def from_horsepower_hours(horsepower_hours: float):
        """
        Create a new instance of Energy from a value in horsepower_hours.

        

        :param meters: The Energy value in horsepower_hours.
        :type horsepower_hours: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(horsepower_hours, EnergyUnits.HorsepowerHour)

    
    @staticmethod
    def from_nanojoules(nanojoules: float):
        """
        Create a new instance of Energy from a value in nanojoules.

        

        :param meters: The Energy value in nanojoules.
        :type nanojoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(nanojoules, EnergyUnits.Nanojoule)

    
    @staticmethod
    def from_microjoules(microjoules: float):
        """
        Create a new instance of Energy from a value in microjoules.

        

        :param meters: The Energy value in microjoules.
        :type microjoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(microjoules, EnergyUnits.Microjoule)

    
    @staticmethod
    def from_millijoules(millijoules: float):
        """
        Create a new instance of Energy from a value in millijoules.

        

        :param meters: The Energy value in millijoules.
        :type millijoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(millijoules, EnergyUnits.Millijoule)

    
    @staticmethod
    def from_kilojoules(kilojoules: float):
        """
        Create a new instance of Energy from a value in kilojoules.

        

        :param meters: The Energy value in kilojoules.
        :type kilojoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(kilojoules, EnergyUnits.Kilojoule)

    
    @staticmethod
    def from_megajoules(megajoules: float):
        """
        Create a new instance of Energy from a value in megajoules.

        

        :param meters: The Energy value in megajoules.
        :type megajoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(megajoules, EnergyUnits.Megajoule)

    
    @staticmethod
    def from_gigajoules(gigajoules: float):
        """
        Create a new instance of Energy from a value in gigajoules.

        

        :param meters: The Energy value in gigajoules.
        :type gigajoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(gigajoules, EnergyUnits.Gigajoule)

    
    @staticmethod
    def from_terajoules(terajoules: float):
        """
        Create a new instance of Energy from a value in terajoules.

        

        :param meters: The Energy value in terajoules.
        :type terajoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(terajoules, EnergyUnits.Terajoule)

    
    @staticmethod
    def from_petajoules(petajoules: float):
        """
        Create a new instance of Energy from a value in petajoules.

        

        :param meters: The Energy value in petajoules.
        :type petajoules: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(petajoules, EnergyUnits.Petajoule)

    
    @staticmethod
    def from_kilocalories(kilocalories: float):
        """
        Create a new instance of Energy from a value in kilocalories.

        

        :param meters: The Energy value in kilocalories.
        :type kilocalories: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(kilocalories, EnergyUnits.Kilocalorie)

    
    @staticmethod
    def from_megacalories(megacalories: float):
        """
        Create a new instance of Energy from a value in megacalories.

        

        :param meters: The Energy value in megacalories.
        :type megacalories: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(megacalories, EnergyUnits.Megacalorie)

    
    @staticmethod
    def from_kilobritish_thermal_units(kilobritish_thermal_units: float):
        """
        Create a new instance of Energy from a value in kilobritish_thermal_units.

        

        :param meters: The Energy value in kilobritish_thermal_units.
        :type kilobritish_thermal_units: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(kilobritish_thermal_units, EnergyUnits.KilobritishThermalUnit)

    
    @staticmethod
    def from_megabritish_thermal_units(megabritish_thermal_units: float):
        """
        Create a new instance of Energy from a value in megabritish_thermal_units.

        

        :param meters: The Energy value in megabritish_thermal_units.
        :type megabritish_thermal_units: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(megabritish_thermal_units, EnergyUnits.MegabritishThermalUnit)

    
    @staticmethod
    def from_gigabritish_thermal_units(gigabritish_thermal_units: float):
        """
        Create a new instance of Energy from a value in gigabritish_thermal_units.

        

        :param meters: The Energy value in gigabritish_thermal_units.
        :type gigabritish_thermal_units: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(gigabritish_thermal_units, EnergyUnits.GigabritishThermalUnit)

    
    @staticmethod
    def from_kiloelectron_volts(kiloelectron_volts: float):
        """
        Create a new instance of Energy from a value in kiloelectron_volts.

        

        :param meters: The Energy value in kiloelectron_volts.
        :type kiloelectron_volts: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(kiloelectron_volts, EnergyUnits.KiloelectronVolt)

    
    @staticmethod
    def from_megaelectron_volts(megaelectron_volts: float):
        """
        Create a new instance of Energy from a value in megaelectron_volts.

        

        :param meters: The Energy value in megaelectron_volts.
        :type megaelectron_volts: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(megaelectron_volts, EnergyUnits.MegaelectronVolt)

    
    @staticmethod
    def from_gigaelectron_volts(gigaelectron_volts: float):
        """
        Create a new instance of Energy from a value in gigaelectron_volts.

        

        :param meters: The Energy value in gigaelectron_volts.
        :type gigaelectron_volts: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(gigaelectron_volts, EnergyUnits.GigaelectronVolt)

    
    @staticmethod
    def from_teraelectron_volts(teraelectron_volts: float):
        """
        Create a new instance of Energy from a value in teraelectron_volts.

        

        :param meters: The Energy value in teraelectron_volts.
        :type teraelectron_volts: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(teraelectron_volts, EnergyUnits.TeraelectronVolt)

    
    @staticmethod
    def from_kilowatt_hours(kilowatt_hours: float):
        """
        Create a new instance of Energy from a value in kilowatt_hours.

        

        :param meters: The Energy value in kilowatt_hours.
        :type kilowatt_hours: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(kilowatt_hours, EnergyUnits.KilowattHour)

    
    @staticmethod
    def from_megawatt_hours(megawatt_hours: float):
        """
        Create a new instance of Energy from a value in megawatt_hours.

        

        :param meters: The Energy value in megawatt_hours.
        :type megawatt_hours: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(megawatt_hours, EnergyUnits.MegawattHour)

    
    @staticmethod
    def from_gigawatt_hours(gigawatt_hours: float):
        """
        Create a new instance of Energy from a value in gigawatt_hours.

        

        :param meters: The Energy value in gigawatt_hours.
        :type gigawatt_hours: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(gigawatt_hours, EnergyUnits.GigawattHour)

    
    @staticmethod
    def from_terawatt_hours(terawatt_hours: float):
        """
        Create a new instance of Energy from a value in terawatt_hours.

        

        :param meters: The Energy value in terawatt_hours.
        :type terawatt_hours: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(terawatt_hours, EnergyUnits.TerawattHour)

    
    @staticmethod
    def from_kilowatt_days(kilowatt_days: float):
        """
        Create a new instance of Energy from a value in kilowatt_days.

        

        :param meters: The Energy value in kilowatt_days.
        :type kilowatt_days: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(kilowatt_days, EnergyUnits.KilowattDay)

    
    @staticmethod
    def from_megawatt_days(megawatt_days: float):
        """
        Create a new instance of Energy from a value in megawatt_days.

        

        :param meters: The Energy value in megawatt_days.
        :type megawatt_days: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(megawatt_days, EnergyUnits.MegawattDay)

    
    @staticmethod
    def from_gigawatt_days(gigawatt_days: float):
        """
        Create a new instance of Energy from a value in gigawatt_days.

        

        :param meters: The Energy value in gigawatt_days.
        :type gigawatt_days: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(gigawatt_days, EnergyUnits.GigawattDay)

    
    @staticmethod
    def from_terawatt_days(terawatt_days: float):
        """
        Create a new instance of Energy from a value in terawatt_days.

        

        :param meters: The Energy value in terawatt_days.
        :type terawatt_days: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(terawatt_days, EnergyUnits.TerawattDay)

    
    @staticmethod
    def from_decatherms_ec(decatherms_ec: float):
        """
        Create a new instance of Energy from a value in decatherms_ec.

        

        :param meters: The Energy value in decatherms_ec.
        :type decatherms_ec: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(decatherms_ec, EnergyUnits.DecathermEc)

    
    @staticmethod
    def from_decatherms_us(decatherms_us: float):
        """
        Create a new instance of Energy from a value in decatherms_us.

        

        :param meters: The Energy value in decatherms_us.
        :type decatherms_us: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(decatherms_us, EnergyUnits.DecathermUs)

    
    @staticmethod
    def from_decatherms_imperial(decatherms_imperial: float):
        """
        Create a new instance of Energy from a value in decatherms_imperial.

        

        :param meters: The Energy value in decatherms_imperial.
        :type decatherms_imperial: float
        :return: A new instance of Energy.
        :rtype: Energy
        """
        return Energy(decatherms_imperial, EnergyUnits.DecathermImperial)

    
    @property
    def joules(self) -> float:
        """
        
        """
        if self.__joules != None:
            return self.__joules
        self.__joules = self.__convert_from_base(EnergyUnits.Joule)
        return self.__joules

    
    @property
    def calories(self) -> float:
        """
        
        """
        if self.__calories != None:
            return self.__calories
        self.__calories = self.__convert_from_base(EnergyUnits.Calorie)
        return self.__calories

    
    @property
    def british_thermal_units(self) -> float:
        """
        
        """
        if self.__british_thermal_units != None:
            return self.__british_thermal_units
        self.__british_thermal_units = self.__convert_from_base(EnergyUnits.BritishThermalUnit)
        return self.__british_thermal_units

    
    @property
    def electron_volts(self) -> float:
        """
        
        """
        if self.__electron_volts != None:
            return self.__electron_volts
        self.__electron_volts = self.__convert_from_base(EnergyUnits.ElectronVolt)
        return self.__electron_volts

    
    @property
    def foot_pounds(self) -> float:
        """
        
        """
        if self.__foot_pounds != None:
            return self.__foot_pounds
        self.__foot_pounds = self.__convert_from_base(EnergyUnits.FootPound)
        return self.__foot_pounds

    
    @property
    def ergs(self) -> float:
        """
        
        """
        if self.__ergs != None:
            return self.__ergs
        self.__ergs = self.__convert_from_base(EnergyUnits.Erg)
        return self.__ergs

    
    @property
    def watt_hours(self) -> float:
        """
        
        """
        if self.__watt_hours != None:
            return self.__watt_hours
        self.__watt_hours = self.__convert_from_base(EnergyUnits.WattHour)
        return self.__watt_hours

    
    @property
    def watt_days(self) -> float:
        """
        
        """
        if self.__watt_days != None:
            return self.__watt_days
        self.__watt_days = self.__convert_from_base(EnergyUnits.WattDay)
        return self.__watt_days

    
    @property
    def therms_ec(self) -> float:
        """
        
        """
        if self.__therms_ec != None:
            return self.__therms_ec
        self.__therms_ec = self.__convert_from_base(EnergyUnits.ThermEc)
        return self.__therms_ec

    
    @property
    def therms_us(self) -> float:
        """
        
        """
        if self.__therms_us != None:
            return self.__therms_us
        self.__therms_us = self.__convert_from_base(EnergyUnits.ThermUs)
        return self.__therms_us

    
    @property
    def therms_imperial(self) -> float:
        """
        
        """
        if self.__therms_imperial != None:
            return self.__therms_imperial
        self.__therms_imperial = self.__convert_from_base(EnergyUnits.ThermImperial)
        return self.__therms_imperial

    
    @property
    def horsepower_hours(self) -> float:
        """
        
        """
        if self.__horsepower_hours != None:
            return self.__horsepower_hours
        self.__horsepower_hours = self.__convert_from_base(EnergyUnits.HorsepowerHour)
        return self.__horsepower_hours

    
    @property
    def nanojoules(self) -> float:
        """
        
        """
        if self.__nanojoules != None:
            return self.__nanojoules
        self.__nanojoules = self.__convert_from_base(EnergyUnits.Nanojoule)
        return self.__nanojoules

    
    @property
    def microjoules(self) -> float:
        """
        
        """
        if self.__microjoules != None:
            return self.__microjoules
        self.__microjoules = self.__convert_from_base(EnergyUnits.Microjoule)
        return self.__microjoules

    
    @property
    def millijoules(self) -> float:
        """
        
        """
        if self.__millijoules != None:
            return self.__millijoules
        self.__millijoules = self.__convert_from_base(EnergyUnits.Millijoule)
        return self.__millijoules

    
    @property
    def kilojoules(self) -> float:
        """
        
        """
        if self.__kilojoules != None:
            return self.__kilojoules
        self.__kilojoules = self.__convert_from_base(EnergyUnits.Kilojoule)
        return self.__kilojoules

    
    @property
    def megajoules(self) -> float:
        """
        
        """
        if self.__megajoules != None:
            return self.__megajoules
        self.__megajoules = self.__convert_from_base(EnergyUnits.Megajoule)
        return self.__megajoules

    
    @property
    def gigajoules(self) -> float:
        """
        
        """
        if self.__gigajoules != None:
            return self.__gigajoules
        self.__gigajoules = self.__convert_from_base(EnergyUnits.Gigajoule)
        return self.__gigajoules

    
    @property
    def terajoules(self) -> float:
        """
        
        """
        if self.__terajoules != None:
            return self.__terajoules
        self.__terajoules = self.__convert_from_base(EnergyUnits.Terajoule)
        return self.__terajoules

    
    @property
    def petajoules(self) -> float:
        """
        
        """
        if self.__petajoules != None:
            return self.__petajoules
        self.__petajoules = self.__convert_from_base(EnergyUnits.Petajoule)
        return self.__petajoules

    
    @property
    def kilocalories(self) -> float:
        """
        
        """
        if self.__kilocalories != None:
            return self.__kilocalories
        self.__kilocalories = self.__convert_from_base(EnergyUnits.Kilocalorie)
        return self.__kilocalories

    
    @property
    def megacalories(self) -> float:
        """
        
        """
        if self.__megacalories != None:
            return self.__megacalories
        self.__megacalories = self.__convert_from_base(EnergyUnits.Megacalorie)
        return self.__megacalories

    
    @property
    def kilobritish_thermal_units(self) -> float:
        """
        
        """
        if self.__kilobritish_thermal_units != None:
            return self.__kilobritish_thermal_units
        self.__kilobritish_thermal_units = self.__convert_from_base(EnergyUnits.KilobritishThermalUnit)
        return self.__kilobritish_thermal_units

    
    @property
    def megabritish_thermal_units(self) -> float:
        """
        
        """
        if self.__megabritish_thermal_units != None:
            return self.__megabritish_thermal_units
        self.__megabritish_thermal_units = self.__convert_from_base(EnergyUnits.MegabritishThermalUnit)
        return self.__megabritish_thermal_units

    
    @property
    def gigabritish_thermal_units(self) -> float:
        """
        
        """
        if self.__gigabritish_thermal_units != None:
            return self.__gigabritish_thermal_units
        self.__gigabritish_thermal_units = self.__convert_from_base(EnergyUnits.GigabritishThermalUnit)
        return self.__gigabritish_thermal_units

    
    @property
    def kiloelectron_volts(self) -> float:
        """
        
        """
        if self.__kiloelectron_volts != None:
            return self.__kiloelectron_volts
        self.__kiloelectron_volts = self.__convert_from_base(EnergyUnits.KiloelectronVolt)
        return self.__kiloelectron_volts

    
    @property
    def megaelectron_volts(self) -> float:
        """
        
        """
        if self.__megaelectron_volts != None:
            return self.__megaelectron_volts
        self.__megaelectron_volts = self.__convert_from_base(EnergyUnits.MegaelectronVolt)
        return self.__megaelectron_volts

    
    @property
    def gigaelectron_volts(self) -> float:
        """
        
        """
        if self.__gigaelectron_volts != None:
            return self.__gigaelectron_volts
        self.__gigaelectron_volts = self.__convert_from_base(EnergyUnits.GigaelectronVolt)
        return self.__gigaelectron_volts

    
    @property
    def teraelectron_volts(self) -> float:
        """
        
        """
        if self.__teraelectron_volts != None:
            return self.__teraelectron_volts
        self.__teraelectron_volts = self.__convert_from_base(EnergyUnits.TeraelectronVolt)
        return self.__teraelectron_volts

    
    @property
    def kilowatt_hours(self) -> float:
        """
        
        """
        if self.__kilowatt_hours != None:
            return self.__kilowatt_hours
        self.__kilowatt_hours = self.__convert_from_base(EnergyUnits.KilowattHour)
        return self.__kilowatt_hours

    
    @property
    def megawatt_hours(self) -> float:
        """
        
        """
        if self.__megawatt_hours != None:
            return self.__megawatt_hours
        self.__megawatt_hours = self.__convert_from_base(EnergyUnits.MegawattHour)
        return self.__megawatt_hours

    
    @property
    def gigawatt_hours(self) -> float:
        """
        
        """
        if self.__gigawatt_hours != None:
            return self.__gigawatt_hours
        self.__gigawatt_hours = self.__convert_from_base(EnergyUnits.GigawattHour)
        return self.__gigawatt_hours

    
    @property
    def terawatt_hours(self) -> float:
        """
        
        """
        if self.__terawatt_hours != None:
            return self.__terawatt_hours
        self.__terawatt_hours = self.__convert_from_base(EnergyUnits.TerawattHour)
        return self.__terawatt_hours

    
    @property
    def kilowatt_days(self) -> float:
        """
        
        """
        if self.__kilowatt_days != None:
            return self.__kilowatt_days
        self.__kilowatt_days = self.__convert_from_base(EnergyUnits.KilowattDay)
        return self.__kilowatt_days

    
    @property
    def megawatt_days(self) -> float:
        """
        
        """
        if self.__megawatt_days != None:
            return self.__megawatt_days
        self.__megawatt_days = self.__convert_from_base(EnergyUnits.MegawattDay)
        return self.__megawatt_days

    
    @property
    def gigawatt_days(self) -> float:
        """
        
        """
        if self.__gigawatt_days != None:
            return self.__gigawatt_days
        self.__gigawatt_days = self.__convert_from_base(EnergyUnits.GigawattDay)
        return self.__gigawatt_days

    
    @property
    def terawatt_days(self) -> float:
        """
        
        """
        if self.__terawatt_days != None:
            return self.__terawatt_days
        self.__terawatt_days = self.__convert_from_base(EnergyUnits.TerawattDay)
        return self.__terawatt_days

    
    @property
    def decatherms_ec(self) -> float:
        """
        
        """
        if self.__decatherms_ec != None:
            return self.__decatherms_ec
        self.__decatherms_ec = self.__convert_from_base(EnergyUnits.DecathermEc)
        return self.__decatherms_ec

    
    @property
    def decatherms_us(self) -> float:
        """
        
        """
        if self.__decatherms_us != None:
            return self.__decatherms_us
        self.__decatherms_us = self.__convert_from_base(EnergyUnits.DecathermUs)
        return self.__decatherms_us

    
    @property
    def decatherms_imperial(self) -> float:
        """
        
        """
        if self.__decatherms_imperial != None:
            return self.__decatherms_imperial
        self.__decatherms_imperial = self.__convert_from_base(EnergyUnits.DecathermImperial)
        return self.__decatherms_imperial

    
    def to_string(self, unit: EnergyUnits = EnergyUnits.Joule, fractional_digits: int = None) -> str:
        """
        Format the Energy to a string.
        
        Note: the default format for Energy is Joule.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Energy. Default is 'Joule'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == EnergyUnits.Joule:
            return f"""{super()._truncate_fraction_digits(self.joules, fractional_digits)} J"""
        
        if unit == EnergyUnits.Calorie:
            return f"""{super()._truncate_fraction_digits(self.calories, fractional_digits)} cal"""
        
        if unit == EnergyUnits.BritishThermalUnit:
            return f"""{super()._truncate_fraction_digits(self.british_thermal_units, fractional_digits)} BTU"""
        
        if unit == EnergyUnits.ElectronVolt:
            return f"""{super()._truncate_fraction_digits(self.electron_volts, fractional_digits)} eV"""
        
        if unit == EnergyUnits.FootPound:
            return f"""{super()._truncate_fraction_digits(self.foot_pounds, fractional_digits)} ft·lb"""
        
        if unit == EnergyUnits.Erg:
            return f"""{super()._truncate_fraction_digits(self.ergs, fractional_digits)} erg"""
        
        if unit == EnergyUnits.WattHour:
            return f"""{super()._truncate_fraction_digits(self.watt_hours, fractional_digits)} Wh"""
        
        if unit == EnergyUnits.WattDay:
            return f"""{super()._truncate_fraction_digits(self.watt_days, fractional_digits)} Wd"""
        
        if unit == EnergyUnits.ThermEc:
            return f"""{super()._truncate_fraction_digits(self.therms_ec, fractional_digits)} th (E.C.)"""
        
        if unit == EnergyUnits.ThermUs:
            return f"""{super()._truncate_fraction_digits(self.therms_us, fractional_digits)} th (U.S.)"""
        
        if unit == EnergyUnits.ThermImperial:
            return f"""{super()._truncate_fraction_digits(self.therms_imperial, fractional_digits)} th (imp.)"""
        
        if unit == EnergyUnits.HorsepowerHour:
            return f"""{super()._truncate_fraction_digits(self.horsepower_hours, fractional_digits)} hp·h"""
        
        if unit == EnergyUnits.Nanojoule:
            return f"""{super()._truncate_fraction_digits(self.nanojoules, fractional_digits)} nJ"""
        
        if unit == EnergyUnits.Microjoule:
            return f"""{super()._truncate_fraction_digits(self.microjoules, fractional_digits)} μJ"""
        
        if unit == EnergyUnits.Millijoule:
            return f"""{super()._truncate_fraction_digits(self.millijoules, fractional_digits)} mJ"""
        
        if unit == EnergyUnits.Kilojoule:
            return f"""{super()._truncate_fraction_digits(self.kilojoules, fractional_digits)} kJ"""
        
        if unit == EnergyUnits.Megajoule:
            return f"""{super()._truncate_fraction_digits(self.megajoules, fractional_digits)} MJ"""
        
        if unit == EnergyUnits.Gigajoule:
            return f"""{super()._truncate_fraction_digits(self.gigajoules, fractional_digits)} GJ"""
        
        if unit == EnergyUnits.Terajoule:
            return f"""{super()._truncate_fraction_digits(self.terajoules, fractional_digits)} TJ"""
        
        if unit == EnergyUnits.Petajoule:
            return f"""{super()._truncate_fraction_digits(self.petajoules, fractional_digits)} PJ"""
        
        if unit == EnergyUnits.Kilocalorie:
            return f"""{super()._truncate_fraction_digits(self.kilocalories, fractional_digits)} kcal"""
        
        if unit == EnergyUnits.Megacalorie:
            return f"""{super()._truncate_fraction_digits(self.megacalories, fractional_digits)} Mcal"""
        
        if unit == EnergyUnits.KilobritishThermalUnit:
            return f"""{super()._truncate_fraction_digits(self.kilobritish_thermal_units, fractional_digits)} kBTU"""
        
        if unit == EnergyUnits.MegabritishThermalUnit:
            return f"""{super()._truncate_fraction_digits(self.megabritish_thermal_units, fractional_digits)} MBTU"""
        
        if unit == EnergyUnits.GigabritishThermalUnit:
            return f"""{super()._truncate_fraction_digits(self.gigabritish_thermal_units, fractional_digits)} GBTU"""
        
        if unit == EnergyUnits.KiloelectronVolt:
            return f"""{super()._truncate_fraction_digits(self.kiloelectron_volts, fractional_digits)} keV"""
        
        if unit == EnergyUnits.MegaelectronVolt:
            return f"""{super()._truncate_fraction_digits(self.megaelectron_volts, fractional_digits)} MeV"""
        
        if unit == EnergyUnits.GigaelectronVolt:
            return f"""{super()._truncate_fraction_digits(self.gigaelectron_volts, fractional_digits)} GeV"""
        
        if unit == EnergyUnits.TeraelectronVolt:
            return f"""{super()._truncate_fraction_digits(self.teraelectron_volts, fractional_digits)} TeV"""
        
        if unit == EnergyUnits.KilowattHour:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_hours, fractional_digits)} kWh"""
        
        if unit == EnergyUnits.MegawattHour:
            return f"""{super()._truncate_fraction_digits(self.megawatt_hours, fractional_digits)} MWh"""
        
        if unit == EnergyUnits.GigawattHour:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_hours, fractional_digits)} GWh"""
        
        if unit == EnergyUnits.TerawattHour:
            return f"""{super()._truncate_fraction_digits(self.terawatt_hours, fractional_digits)} TWh"""
        
        if unit == EnergyUnits.KilowattDay:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_days, fractional_digits)} kWd"""
        
        if unit == EnergyUnits.MegawattDay:
            return f"""{super()._truncate_fraction_digits(self.megawatt_days, fractional_digits)} MWd"""
        
        if unit == EnergyUnits.GigawattDay:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_days, fractional_digits)} GWd"""
        
        if unit == EnergyUnits.TerawattDay:
            return f"""{super()._truncate_fraction_digits(self.terawatt_days, fractional_digits)} TWd"""
        
        if unit == EnergyUnits.DecathermEc:
            return f"""{super()._truncate_fraction_digits(self.decatherms_ec, fractional_digits)} dath (E.C.)"""
        
        if unit == EnergyUnits.DecathermUs:
            return f"""{super()._truncate_fraction_digits(self.decatherms_us, fractional_digits)} dath (U.S.)"""
        
        if unit == EnergyUnits.DecathermImperial:
            return f"""{super()._truncate_fraction_digits(self.decatherms_imperial, fractional_digits)} dath (imp.)"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: EnergyUnits = EnergyUnits.Joule) -> str:
        """
        Get Energy unit abbreviation.
        Note! the default abbreviation for Energy is Joule.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == EnergyUnits.Joule:
            return """J"""
        
        if unit_abbreviation == EnergyUnits.Calorie:
            return """cal"""
        
        if unit_abbreviation == EnergyUnits.BritishThermalUnit:
            return """BTU"""
        
        if unit_abbreviation == EnergyUnits.ElectronVolt:
            return """eV"""
        
        if unit_abbreviation == EnergyUnits.FootPound:
            return """ft·lb"""
        
        if unit_abbreviation == EnergyUnits.Erg:
            return """erg"""
        
        if unit_abbreviation == EnergyUnits.WattHour:
            return """Wh"""
        
        if unit_abbreviation == EnergyUnits.WattDay:
            return """Wd"""
        
        if unit_abbreviation == EnergyUnits.ThermEc:
            return """th (E.C.)"""
        
        if unit_abbreviation == EnergyUnits.ThermUs:
            return """th (U.S.)"""
        
        if unit_abbreviation == EnergyUnits.ThermImperial:
            return """th (imp.)"""
        
        if unit_abbreviation == EnergyUnits.HorsepowerHour:
            return """hp·h"""
        
        if unit_abbreviation == EnergyUnits.Nanojoule:
            return """nJ"""
        
        if unit_abbreviation == EnergyUnits.Microjoule:
            return """μJ"""
        
        if unit_abbreviation == EnergyUnits.Millijoule:
            return """mJ"""
        
        if unit_abbreviation == EnergyUnits.Kilojoule:
            return """kJ"""
        
        if unit_abbreviation == EnergyUnits.Megajoule:
            return """MJ"""
        
        if unit_abbreviation == EnergyUnits.Gigajoule:
            return """GJ"""
        
        if unit_abbreviation == EnergyUnits.Terajoule:
            return """TJ"""
        
        if unit_abbreviation == EnergyUnits.Petajoule:
            return """PJ"""
        
        if unit_abbreviation == EnergyUnits.Kilocalorie:
            return """kcal"""
        
        if unit_abbreviation == EnergyUnits.Megacalorie:
            return """Mcal"""
        
        if unit_abbreviation == EnergyUnits.KilobritishThermalUnit:
            return """kBTU"""
        
        if unit_abbreviation == EnergyUnits.MegabritishThermalUnit:
            return """MBTU"""
        
        if unit_abbreviation == EnergyUnits.GigabritishThermalUnit:
            return """GBTU"""
        
        if unit_abbreviation == EnergyUnits.KiloelectronVolt:
            return """keV"""
        
        if unit_abbreviation == EnergyUnits.MegaelectronVolt:
            return """MeV"""
        
        if unit_abbreviation == EnergyUnits.GigaelectronVolt:
            return """GeV"""
        
        if unit_abbreviation == EnergyUnits.TeraelectronVolt:
            return """TeV"""
        
        if unit_abbreviation == EnergyUnits.KilowattHour:
            return """kWh"""
        
        if unit_abbreviation == EnergyUnits.MegawattHour:
            return """MWh"""
        
        if unit_abbreviation == EnergyUnits.GigawattHour:
            return """GWh"""
        
        if unit_abbreviation == EnergyUnits.TerawattHour:
            return """TWh"""
        
        if unit_abbreviation == EnergyUnits.KilowattDay:
            return """kWd"""
        
        if unit_abbreviation == EnergyUnits.MegawattDay:
            return """MWd"""
        
        if unit_abbreviation == EnergyUnits.GigawattDay:
            return """GWd"""
        
        if unit_abbreviation == EnergyUnits.TerawattDay:
            return """TWd"""
        
        if unit_abbreviation == EnergyUnits.DecathermEc:
            return """dath (E.C.)"""
        
        if unit_abbreviation == EnergyUnits.DecathermUs:
            return """dath (U.S.)"""
        
        if unit_abbreviation == EnergyUnits.DecathermImperial:
            return """dath (imp.)"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class PermeabilityUnits(Enum):
        """
            PermeabilityUnits enumeration
        """
        
        HenryPerMeter = 'HenryPerMeter'
        """
            
        """
        

class PermeabilityDto:
    """
    A DTO representation of a Permeability

    Attributes:
        value (float): The value of the Permeability.
        unit (PermeabilityUnits): The specific unit that the Permeability value is representing.
    """

    def __init__(self, value: float, unit: PermeabilityUnits):
        """
        Create a new DTO representation of a Permeability

        Parameters:
            value (float): The value of the Permeability.
            unit (PermeabilityUnits): The specific unit that the Permeability value is representing.
        """
        self.value: float = value
        """
        The value of the Permeability
        """
        self.unit: PermeabilityUnits = unit
        """
        The specific unit that the Permeability value is representing
        """

    def to_json(self):
        """
        Get a Permeability DTO JSON object representing the current unit.

        :return: JSON object represents Permeability DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "HenryPerMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Permeability DTO from a json representation.

        :param data: The Permeability DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "HenryPerMeter"}
        :return: A new instance of PermeabilityDto.
        :rtype: PermeabilityDto
        """
        return PermeabilityDto(value=data["value"], unit=PermeabilityUnits(data["unit"]))


class Permeability(AbstractMeasure):
    """
    In electromagnetism, permeability is the measure of the ability of a material to support the formation of a magnetic field within itself.

    Args:
        value (float): The value.
        from_unit (PermeabilityUnits): The Permeability unit to create from, The default unit is HenryPerMeter
    """
    def __init__(self, value: float, from_unit: PermeabilityUnits = PermeabilityUnits.HenryPerMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__henries_per_meter = None
        

    def convert(self, unit: PermeabilityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: PermeabilityUnits = PermeabilityUnits.HenryPerMeter) -> PermeabilityDto:
        """
        Get a new instance of Permeability DTO representing the current unit.

        :param hold_in_unit: The specific Permeability unit to store the Permeability value in the DTO representation.
        :type hold_in_unit: PermeabilityUnits
        :return: A new instance of PermeabilityDto.
        :rtype: PermeabilityDto
        """
        return PermeabilityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: PermeabilityUnits = PermeabilityUnits.HenryPerMeter):
        """
        Get a Permeability DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Permeability unit to store the Permeability value in the DTO representation.
        :type hold_in_unit: PermeabilityUnits
        :return: JSON object represents Permeability DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "HenryPerMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(permeability_dto: PermeabilityDto):
        """
        Obtain a new instance of Permeability from a DTO unit object.

        :param permeability_dto: The Permeability DTO representation.
        :type permeability_dto: PermeabilityDto
        :return: A new instance of Permeability.
        :rtype: Permeability
        """
        return Permeability(permeability_dto.value, permeability_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Permeability from a DTO unit json representation.

        :param data: The Permeability DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "HenryPerMeter"}
        :return: A new instance of Permeability.
        :rtype: Permeability
        """
        return Permeability.from_dto(PermeabilityDto.from_json(data))

    def __convert_from_base(self, from_unit: PermeabilityUnits) -> float:
        value = self._value
        
        if from_unit == PermeabilityUnits.HenryPerMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PermeabilityUnits) -> float:
        
        if to_unit == PermeabilityUnits.HenryPerMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_henries_per_meter(henries_per_meter: float):
        """
        Create a new instance of Permeability from a value in henries_per_meter.

        

        :param meters: The Permeability value in henries_per_meter.
        :type henries_per_meter: float
        :return: A new instance of Permeability.
        :rtype: Permeability
        """
        return Permeability(henries_per_meter, PermeabilityUnits.HenryPerMeter)

    
    @property
    def henries_per_meter(self) -> float:
        """
        
        """
        if self.__henries_per_meter != None:
            return self.__henries_per_meter
        self.__henries_per_meter = self.__convert_from_base(PermeabilityUnits.HenryPerMeter)
        return self.__henries_per_meter

    
    def to_string(self, unit: PermeabilityUnits = PermeabilityUnits.HenryPerMeter, fractional_digits: int = None) -> str:
        """
        Format the Permeability to a string.
        
        Note: the default format for Permeability is HenryPerMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Permeability. Default is 'HenryPerMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == PermeabilityUnits.HenryPerMeter:
            return f"""{super()._truncate_fraction_digits(self.henries_per_meter, fractional_digits)} H/m"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: PermeabilityUnits = PermeabilityUnits.HenryPerMeter) -> str:
        """
        Get Permeability unit abbreviation.
        Note! the default abbreviation for Permeability is HenryPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PermeabilityUnits.HenryPerMeter:
            return """H/m"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class PowerUnits(Enum):
        """
            PowerUnits enumeration
        """
        
        Watt = 'Watt'
        """
            
        """
        
        MechanicalHorsepower = 'MechanicalHorsepower'
        """
            
        """
        
        MetricHorsepower = 'MetricHorsepower'
        """
            
        """
        
        ElectricalHorsepower = 'ElectricalHorsepower'
        """
            
        """
        
        BoilerHorsepower = 'BoilerHorsepower'
        """
            
        """
        
        HydraulicHorsepower = 'HydraulicHorsepower'
        """
            
        """
        
        BritishThermalUnitPerHour = 'BritishThermalUnitPerHour'
        """
            
        """
        
        JoulePerHour = 'JoulePerHour'
        """
            
        """
        
        TonOfRefrigeration = 'TonOfRefrigeration'
        """
            
        """
        
        Femtowatt = 'Femtowatt'
        """
            
        """
        
        Picowatt = 'Picowatt'
        """
            
        """
        
        Nanowatt = 'Nanowatt'
        """
            
        """
        
        Microwatt = 'Microwatt'
        """
            
        """
        
        Milliwatt = 'Milliwatt'
        """
            
        """
        
        Deciwatt = 'Deciwatt'
        """
            
        """
        
        Decawatt = 'Decawatt'
        """
            
        """
        
        Kilowatt = 'Kilowatt'
        """
            
        """
        
        Megawatt = 'Megawatt'
        """
            
        """
        
        Gigawatt = 'Gigawatt'
        """
            
        """
        
        Terawatt = 'Terawatt'
        """
            
        """
        
        Petawatt = 'Petawatt'
        """
            
        """
        
        KilobritishThermalUnitPerHour = 'KilobritishThermalUnitPerHour'
        """
            
        """
        
        MegabritishThermalUnitPerHour = 'MegabritishThermalUnitPerHour'
        """
            
        """
        
        MillijoulePerHour = 'MillijoulePerHour'
        """
            
        """
        
        KilojoulePerHour = 'KilojoulePerHour'
        """
            
        """
        
        MegajoulePerHour = 'MegajoulePerHour'
        """
            
        """
        
        GigajoulePerHour = 'GigajoulePerHour'
        """
            
        """
        

class PowerDto:
    """
    A DTO representation of a Power

    Attributes:
        value (float): The value of the Power.
        unit (PowerUnits): The specific unit that the Power value is representing.
    """

    def __init__(self, value: float, unit: PowerUnits):
        """
        Create a new DTO representation of a Power

        Parameters:
            value (float): The value of the Power.
            unit (PowerUnits): The specific unit that the Power value is representing.
        """
        self.value: float = value
        """
        The value of the Power
        """
        self.unit: PowerUnits = unit
        """
        The specific unit that the Power value is representing
        """

    def to_json(self):
        """
        Get a Power DTO JSON object representing the current unit.

        :return: JSON object represents Power DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Watt"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Power DTO from a json representation.

        :param data: The Power DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Watt"}
        :return: A new instance of PowerDto.
        :rtype: PowerDto
        """
        return PowerDto(value=data["value"], unit=PowerUnits(data["unit"]))


class Power(AbstractMeasure):
    """
    In physics, power is the rate of doing work. It is equivalent to an amount of energy consumed per unit time.

    Args:
        value (float): The value.
        from_unit (PowerUnits): The Power unit to create from, The default unit is Watt
    """
    def __init__(self, value: float, from_unit: PowerUnits = PowerUnits.Watt):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__watts = None
        
        self.__mechanical_horsepower = None
        
        self.__metric_horsepower = None
        
        self.__electrical_horsepower = None
        
        self.__boiler_horsepower = None
        
        self.__hydraulic_horsepower = None
        
        self.__british_thermal_units_per_hour = None
        
        self.__joules_per_hour = None
        
        self.__tons_of_refrigeration = None
        
        self.__femtowatts = None
        
        self.__picowatts = None
        
        self.__nanowatts = None
        
        self.__microwatts = None
        
        self.__milliwatts = None
        
        self.__deciwatts = None
        
        self.__decawatts = None
        
        self.__kilowatts = None
        
        self.__megawatts = None
        
        self.__gigawatts = None
        
        self.__terawatts = None
        
        self.__petawatts = None
        
        self.__kilobritish_thermal_units_per_hour = None
        
        self.__megabritish_thermal_units_per_hour = None
        
        self.__millijoules_per_hour = None
        
        self.__kilojoules_per_hour = None
        
        self.__megajoules_per_hour = None
        
        self.__gigajoules_per_hour = None
        

    def convert(self, unit: PowerUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: PowerUnits = PowerUnits.Watt) -> PowerDto:
        """
        Get a new instance of Power DTO representing the current unit.

        :param hold_in_unit: The specific Power unit to store the Power value in the DTO representation.
        :type hold_in_unit: PowerUnits
        :return: A new instance of PowerDto.
        :rtype: PowerDto
        """
        return PowerDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: PowerUnits = PowerUnits.Watt):
        """
        Get a Power DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Power unit to store the Power value in the DTO representation.
        :type hold_in_unit: PowerUnits
        :return: JSON object represents Power DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Watt"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(power_dto: PowerDto):
        """
        Obtain a new instance of Power from a DTO unit object.

        :param power_dto: The Power DTO representation.
        :type power_dto: PowerDto
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(power_dto.value, power_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Power from a DTO unit json representation.

        :param data: The Power DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Watt"}
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power.from_dto(PowerDto.from_json(data))

    def __convert_from_base(self, from_unit: PowerUnits) -> float:
        value = self._value
        
        if from_unit == PowerUnits.Watt:
            return (value)
        
        if from_unit == PowerUnits.MechanicalHorsepower:
            return (value / 745.69)
        
        if from_unit == PowerUnits.MetricHorsepower:
            return (value / 735.49875)
        
        if from_unit == PowerUnits.ElectricalHorsepower:
            return (value / 746)
        
        if from_unit == PowerUnits.BoilerHorsepower:
            return (value / 9812.5)
        
        if from_unit == PowerUnits.HydraulicHorsepower:
            return (value / 745.69988145)
        
        if from_unit == PowerUnits.BritishThermalUnitPerHour:
            return (value / 0.29307107017)
        
        if from_unit == PowerUnits.JoulePerHour:
            return (value * 3600)
        
        if from_unit == PowerUnits.TonOfRefrigeration:
            return (value / 3516.853)
        
        if from_unit == PowerUnits.Femtowatt:
            return ((value) / 1e-15)
        
        if from_unit == PowerUnits.Picowatt:
            return ((value) / 1e-12)
        
        if from_unit == PowerUnits.Nanowatt:
            return ((value) / 1e-09)
        
        if from_unit == PowerUnits.Microwatt:
            return ((value) / 1e-06)
        
        if from_unit == PowerUnits.Milliwatt:
            return ((value) / 0.001)
        
        if from_unit == PowerUnits.Deciwatt:
            return ((value) / 0.1)
        
        if from_unit == PowerUnits.Decawatt:
            return ((value) / 10.0)
        
        if from_unit == PowerUnits.Kilowatt:
            return ((value) / 1000.0)
        
        if from_unit == PowerUnits.Megawatt:
            return ((value) / 1000000.0)
        
        if from_unit == PowerUnits.Gigawatt:
            return ((value) / 1000000000.0)
        
        if from_unit == PowerUnits.Terawatt:
            return ((value) / 1000000000000.0)
        
        if from_unit == PowerUnits.Petawatt:
            return ((value) / 1000000000000000.0)
        
        if from_unit == PowerUnits.KilobritishThermalUnitPerHour:
            return ((value / 0.29307107017) / 1000.0)
        
        if from_unit == PowerUnits.MegabritishThermalUnitPerHour:
            return ((value / 0.29307107017) / 1000000.0)
        
        if from_unit == PowerUnits.MillijoulePerHour:
            return ((value * 3600) / 0.001)
        
        if from_unit == PowerUnits.KilojoulePerHour:
            return ((value * 3600) / 1000.0)
        
        if from_unit == PowerUnits.MegajoulePerHour:
            return ((value * 3600) / 1000000.0)
        
        if from_unit == PowerUnits.GigajoulePerHour:
            return ((value * 3600) / 1000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PowerUnits) -> float:
        
        if to_unit == PowerUnits.Watt:
            return (value)
        
        if to_unit == PowerUnits.MechanicalHorsepower:
            return (value * 745.69)
        
        if to_unit == PowerUnits.MetricHorsepower:
            return (value * 735.49875)
        
        if to_unit == PowerUnits.ElectricalHorsepower:
            return (value * 746)
        
        if to_unit == PowerUnits.BoilerHorsepower:
            return (value * 9812.5)
        
        if to_unit == PowerUnits.HydraulicHorsepower:
            return (value * 745.69988145)
        
        if to_unit == PowerUnits.BritishThermalUnitPerHour:
            return (value * 0.29307107017)
        
        if to_unit == PowerUnits.JoulePerHour:
            return (value / 3600)
        
        if to_unit == PowerUnits.TonOfRefrigeration:
            return (value * 3516.853)
        
        if to_unit == PowerUnits.Femtowatt:
            return ((value) * 1e-15)
        
        if to_unit == PowerUnits.Picowatt:
            return ((value) * 1e-12)
        
        if to_unit == PowerUnits.Nanowatt:
            return ((value) * 1e-09)
        
        if to_unit == PowerUnits.Microwatt:
            return ((value) * 1e-06)
        
        if to_unit == PowerUnits.Milliwatt:
            return ((value) * 0.001)
        
        if to_unit == PowerUnits.Deciwatt:
            return ((value) * 0.1)
        
        if to_unit == PowerUnits.Decawatt:
            return ((value) * 10.0)
        
        if to_unit == PowerUnits.Kilowatt:
            return ((value) * 1000.0)
        
        if to_unit == PowerUnits.Megawatt:
            return ((value) * 1000000.0)
        
        if to_unit == PowerUnits.Gigawatt:
            return ((value) * 1000000000.0)
        
        if to_unit == PowerUnits.Terawatt:
            return ((value) * 1000000000000.0)
        
        if to_unit == PowerUnits.Petawatt:
            return ((value) * 1000000000000000.0)
        
        if to_unit == PowerUnits.KilobritishThermalUnitPerHour:
            return ((value * 0.29307107017) * 1000.0)
        
        if to_unit == PowerUnits.MegabritishThermalUnitPerHour:
            return ((value * 0.29307107017) * 1000000.0)
        
        if to_unit == PowerUnits.MillijoulePerHour:
            return ((value / 3600) * 0.001)
        
        if to_unit == PowerUnits.KilojoulePerHour:
            return ((value / 3600) * 1000.0)
        
        if to_unit == PowerUnits.MegajoulePerHour:
            return ((value / 3600) * 1000000.0)
        
        if to_unit == PowerUnits.GigajoulePerHour:
            return ((value / 3600) * 1000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_watts(watts: float):
        """
        Create a new instance of Power from a value in watts.

        

        :param meters: The Power value in watts.
        :type watts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(watts, PowerUnits.Watt)

    
    @staticmethod
    def from_mechanical_horsepower(mechanical_horsepower: float):
        """
        Create a new instance of Power from a value in mechanical_horsepower.

        

        :param meters: The Power value in mechanical_horsepower.
        :type mechanical_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(mechanical_horsepower, PowerUnits.MechanicalHorsepower)

    
    @staticmethod
    def from_metric_horsepower(metric_horsepower: float):
        """
        Create a new instance of Power from a value in metric_horsepower.

        

        :param meters: The Power value in metric_horsepower.
        :type metric_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(metric_horsepower, PowerUnits.MetricHorsepower)

    
    @staticmethod
    def from_electrical_horsepower(electrical_horsepower: float):
        """
        Create a new instance of Power from a value in electrical_horsepower.

        

        :param meters: The Power value in electrical_horsepower.
        :type electrical_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(electrical_horsepower, PowerUnits.ElectricalHorsepower)

    
    @staticmethod
    def from_boiler_horsepower(boiler_horsepower: float):
        """
        Create a new instance of Power from a value in boiler_horsepower.

        

        :param meters: The Power value in boiler_horsepower.
        :type boiler_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(boiler_horsepower, PowerUnits.BoilerHorsepower)

    
    @staticmethod
    def from_hydraulic_horsepower(hydraulic_horsepower: float):
        """
        Create a new instance of Power from a value in hydraulic_horsepower.

        

        :param meters: The Power value in hydraulic_horsepower.
        :type hydraulic_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(hydraulic_horsepower, PowerUnits.HydraulicHorsepower)

    
    @staticmethod
    def from_british_thermal_units_per_hour(british_thermal_units_per_hour: float):
        """
        Create a new instance of Power from a value in british_thermal_units_per_hour.

        

        :param meters: The Power value in british_thermal_units_per_hour.
        :type british_thermal_units_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(british_thermal_units_per_hour, PowerUnits.BritishThermalUnitPerHour)

    
    @staticmethod
    def from_joules_per_hour(joules_per_hour: float):
        """
        Create a new instance of Power from a value in joules_per_hour.

        

        :param meters: The Power value in joules_per_hour.
        :type joules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(joules_per_hour, PowerUnits.JoulePerHour)

    
    @staticmethod
    def from_tons_of_refrigeration(tons_of_refrigeration: float):
        """
        Create a new instance of Power from a value in tons_of_refrigeration.

        

        :param meters: The Power value in tons_of_refrigeration.
        :type tons_of_refrigeration: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(tons_of_refrigeration, PowerUnits.TonOfRefrigeration)

    
    @staticmethod
    def from_femtowatts(femtowatts: float):
        """
        Create a new instance of Power from a value in femtowatts.

        

        :param meters: The Power value in femtowatts.
        :type femtowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(femtowatts, PowerUnits.Femtowatt)

    
    @staticmethod
    def from_picowatts(picowatts: float):
        """
        Create a new instance of Power from a value in picowatts.

        

        :param meters: The Power value in picowatts.
        :type picowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(picowatts, PowerUnits.Picowatt)

    
    @staticmethod
    def from_nanowatts(nanowatts: float):
        """
        Create a new instance of Power from a value in nanowatts.

        

        :param meters: The Power value in nanowatts.
        :type nanowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(nanowatts, PowerUnits.Nanowatt)

    
    @staticmethod
    def from_microwatts(microwatts: float):
        """
        Create a new instance of Power from a value in microwatts.

        

        :param meters: The Power value in microwatts.
        :type microwatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(microwatts, PowerUnits.Microwatt)

    
    @staticmethod
    def from_milliwatts(milliwatts: float):
        """
        Create a new instance of Power from a value in milliwatts.

        

        :param meters: The Power value in milliwatts.
        :type milliwatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(milliwatts, PowerUnits.Milliwatt)

    
    @staticmethod
    def from_deciwatts(deciwatts: float):
        """
        Create a new instance of Power from a value in deciwatts.

        

        :param meters: The Power value in deciwatts.
        :type deciwatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(deciwatts, PowerUnits.Deciwatt)

    
    @staticmethod
    def from_decawatts(decawatts: float):
        """
        Create a new instance of Power from a value in decawatts.

        

        :param meters: The Power value in decawatts.
        :type decawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(decawatts, PowerUnits.Decawatt)

    
    @staticmethod
    def from_kilowatts(kilowatts: float):
        """
        Create a new instance of Power from a value in kilowatts.

        

        :param meters: The Power value in kilowatts.
        :type kilowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(kilowatts, PowerUnits.Kilowatt)

    
    @staticmethod
    def from_megawatts(megawatts: float):
        """
        Create a new instance of Power from a value in megawatts.

        

        :param meters: The Power value in megawatts.
        :type megawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(megawatts, PowerUnits.Megawatt)

    
    @staticmethod
    def from_gigawatts(gigawatts: float):
        """
        Create a new instance of Power from a value in gigawatts.

        

        :param meters: The Power value in gigawatts.
        :type gigawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(gigawatts, PowerUnits.Gigawatt)

    
    @staticmethod
    def from_terawatts(terawatts: float):
        """
        Create a new instance of Power from a value in terawatts.

        

        :param meters: The Power value in terawatts.
        :type terawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(terawatts, PowerUnits.Terawatt)

    
    @staticmethod
    def from_petawatts(petawatts: float):
        """
        Create a new instance of Power from a value in petawatts.

        

        :param meters: The Power value in petawatts.
        :type petawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(petawatts, PowerUnits.Petawatt)

    
    @staticmethod
    def from_kilobritish_thermal_units_per_hour(kilobritish_thermal_units_per_hour: float):
        """
        Create a new instance of Power from a value in kilobritish_thermal_units_per_hour.

        

        :param meters: The Power value in kilobritish_thermal_units_per_hour.
        :type kilobritish_thermal_units_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(kilobritish_thermal_units_per_hour, PowerUnits.KilobritishThermalUnitPerHour)

    
    @staticmethod
    def from_megabritish_thermal_units_per_hour(megabritish_thermal_units_per_hour: float):
        """
        Create a new instance of Power from a value in megabritish_thermal_units_per_hour.

        

        :param meters: The Power value in megabritish_thermal_units_per_hour.
        :type megabritish_thermal_units_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(megabritish_thermal_units_per_hour, PowerUnits.MegabritishThermalUnitPerHour)

    
    @staticmethod
    def from_millijoules_per_hour(millijoules_per_hour: float):
        """
        Create a new instance of Power from a value in millijoules_per_hour.

        

        :param meters: The Power value in millijoules_per_hour.
        :type millijoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(millijoules_per_hour, PowerUnits.MillijoulePerHour)

    
    @staticmethod
    def from_kilojoules_per_hour(kilojoules_per_hour: float):
        """
        Create a new instance of Power from a value in kilojoules_per_hour.

        

        :param meters: The Power value in kilojoules_per_hour.
        :type kilojoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(kilojoules_per_hour, PowerUnits.KilojoulePerHour)

    
    @staticmethod
    def from_megajoules_per_hour(megajoules_per_hour: float):
        """
        Create a new instance of Power from a value in megajoules_per_hour.

        

        :param meters: The Power value in megajoules_per_hour.
        :type megajoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(megajoules_per_hour, PowerUnits.MegajoulePerHour)

    
    @staticmethod
    def from_gigajoules_per_hour(gigajoules_per_hour: float):
        """
        Create a new instance of Power from a value in gigajoules_per_hour.

        

        :param meters: The Power value in gigajoules_per_hour.
        :type gigajoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(gigajoules_per_hour, PowerUnits.GigajoulePerHour)

    
    @property
    def watts(self) -> float:
        """
        
        """
        if self.__watts != None:
            return self.__watts
        self.__watts = self.__convert_from_base(PowerUnits.Watt)
        return self.__watts

    
    @property
    def mechanical_horsepower(self) -> float:
        """
        
        """
        if self.__mechanical_horsepower != None:
            return self.__mechanical_horsepower
        self.__mechanical_horsepower = self.__convert_from_base(PowerUnits.MechanicalHorsepower)
        return self.__mechanical_horsepower

    
    @property
    def metric_horsepower(self) -> float:
        """
        
        """
        if self.__metric_horsepower != None:
            return self.__metric_horsepower
        self.__metric_horsepower = self.__convert_from_base(PowerUnits.MetricHorsepower)
        return self.__metric_horsepower

    
    @property
    def electrical_horsepower(self) -> float:
        """
        
        """
        if self.__electrical_horsepower != None:
            return self.__electrical_horsepower
        self.__electrical_horsepower = self.__convert_from_base(PowerUnits.ElectricalHorsepower)
        return self.__electrical_horsepower

    
    @property
    def boiler_horsepower(self) -> float:
        """
        
        """
        if self.__boiler_horsepower != None:
            return self.__boiler_horsepower
        self.__boiler_horsepower = self.__convert_from_base(PowerUnits.BoilerHorsepower)
        return self.__boiler_horsepower

    
    @property
    def hydraulic_horsepower(self) -> float:
        """
        
        """
        if self.__hydraulic_horsepower != None:
            return self.__hydraulic_horsepower
        self.__hydraulic_horsepower = self.__convert_from_base(PowerUnits.HydraulicHorsepower)
        return self.__hydraulic_horsepower

    
    @property
    def british_thermal_units_per_hour(self) -> float:
        """
        
        """
        if self.__british_thermal_units_per_hour != None:
            return self.__british_thermal_units_per_hour
        self.__british_thermal_units_per_hour = self.__convert_from_base(PowerUnits.BritishThermalUnitPerHour)
        return self.__british_thermal_units_per_hour

    
    @property
    def joules_per_hour(self) -> float:
        """
        
        """
        if self.__joules_per_hour != None:
            return self.__joules_per_hour
        self.__joules_per_hour = self.__convert_from_base(PowerUnits.JoulePerHour)
        return self.__joules_per_hour

    
    @property
    def tons_of_refrigeration(self) -> float:
        """
        
        """
        if self.__tons_of_refrigeration != None:
            return self.__tons_of_refrigeration
        self.__tons_of_refrigeration = self.__convert_from_base(PowerUnits.TonOfRefrigeration)
        return self.__tons_of_refrigeration

    
    @property
    def femtowatts(self) -> float:
        """
        
        """
        if self.__femtowatts != None:
            return self.__femtowatts
        self.__femtowatts = self.__convert_from_base(PowerUnits.Femtowatt)
        return self.__femtowatts

    
    @property
    def picowatts(self) -> float:
        """
        
        """
        if self.__picowatts != None:
            return self.__picowatts
        self.__picowatts = self.__convert_from_base(PowerUnits.Picowatt)
        return self.__picowatts

    
    @property
    def nanowatts(self) -> float:
        """
        
        """
        if self.__nanowatts != None:
            return self.__nanowatts
        self.__nanowatts = self.__convert_from_base(PowerUnits.Nanowatt)
        return self.__nanowatts

    
    @property
    def microwatts(self) -> float:
        """
        
        """
        if self.__microwatts != None:
            return self.__microwatts
        self.__microwatts = self.__convert_from_base(PowerUnits.Microwatt)
        return self.__microwatts

    
    @property
    def milliwatts(self) -> float:
        """
        
        """
        if self.__milliwatts != None:
            return self.__milliwatts
        self.__milliwatts = self.__convert_from_base(PowerUnits.Milliwatt)
        return self.__milliwatts

    
    @property
    def deciwatts(self) -> float:
        """
        
        """
        if self.__deciwatts != None:
            return self.__deciwatts
        self.__deciwatts = self.__convert_from_base(PowerUnits.Deciwatt)
        return self.__deciwatts

    
    @property
    def decawatts(self) -> float:
        """
        
        """
        if self.__decawatts != None:
            return self.__decawatts
        self.__decawatts = self.__convert_from_base(PowerUnits.Decawatt)
        return self.__decawatts

    
    @property
    def kilowatts(self) -> float:
        """
        
        """
        if self.__kilowatts != None:
            return self.__kilowatts
        self.__kilowatts = self.__convert_from_base(PowerUnits.Kilowatt)
        return self.__kilowatts

    
    @property
    def megawatts(self) -> float:
        """
        
        """
        if self.__megawatts != None:
            return self.__megawatts
        self.__megawatts = self.__convert_from_base(PowerUnits.Megawatt)
        return self.__megawatts

    
    @property
    def gigawatts(self) -> float:
        """
        
        """
        if self.__gigawatts != None:
            return self.__gigawatts
        self.__gigawatts = self.__convert_from_base(PowerUnits.Gigawatt)
        return self.__gigawatts

    
    @property
    def terawatts(self) -> float:
        """
        
        """
        if self.__terawatts != None:
            return self.__terawatts
        self.__terawatts = self.__convert_from_base(PowerUnits.Terawatt)
        return self.__terawatts

    
    @property
    def petawatts(self) -> float:
        """
        
        """
        if self.__petawatts != None:
            return self.__petawatts
        self.__petawatts = self.__convert_from_base(PowerUnits.Petawatt)
        return self.__petawatts

    
    @property
    def kilobritish_thermal_units_per_hour(self) -> float:
        """
        
        """
        if self.__kilobritish_thermal_units_per_hour != None:
            return self.__kilobritish_thermal_units_per_hour
        self.__kilobritish_thermal_units_per_hour = self.__convert_from_base(PowerUnits.KilobritishThermalUnitPerHour)
        return self.__kilobritish_thermal_units_per_hour

    
    @property
    def megabritish_thermal_units_per_hour(self) -> float:
        """
        
        """
        if self.__megabritish_thermal_units_per_hour != None:
            return self.__megabritish_thermal_units_per_hour
        self.__megabritish_thermal_units_per_hour = self.__convert_from_base(PowerUnits.MegabritishThermalUnitPerHour)
        return self.__megabritish_thermal_units_per_hour

    
    @property
    def millijoules_per_hour(self) -> float:
        """
        
        """
        if self.__millijoules_per_hour != None:
            return self.__millijoules_per_hour
        self.__millijoules_per_hour = self.__convert_from_base(PowerUnits.MillijoulePerHour)
        return self.__millijoules_per_hour

    
    @property
    def kilojoules_per_hour(self) -> float:
        """
        
        """
        if self.__kilojoules_per_hour != None:
            return self.__kilojoules_per_hour
        self.__kilojoules_per_hour = self.__convert_from_base(PowerUnits.KilojoulePerHour)
        return self.__kilojoules_per_hour

    
    @property
    def megajoules_per_hour(self) -> float:
        """
        
        """
        if self.__megajoules_per_hour != None:
            return self.__megajoules_per_hour
        self.__megajoules_per_hour = self.__convert_from_base(PowerUnits.MegajoulePerHour)
        return self.__megajoules_per_hour

    
    @property
    def gigajoules_per_hour(self) -> float:
        """
        
        """
        if self.__gigajoules_per_hour != None:
            return self.__gigajoules_per_hour
        self.__gigajoules_per_hour = self.__convert_from_base(PowerUnits.GigajoulePerHour)
        return self.__gigajoules_per_hour

    
    def to_string(self, unit: PowerUnits = PowerUnits.Watt, fractional_digits: int = None) -> str:
        """
        Format the Power to a string.
        
        Note: the default format for Power is Watt.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Power. Default is 'Watt'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == PowerUnits.Watt:
            return f"""{super()._truncate_fraction_digits(self.watts, fractional_digits)} W"""
        
        if unit == PowerUnits.MechanicalHorsepower:
            return f"""{super()._truncate_fraction_digits(self.mechanical_horsepower, fractional_digits)} hp(I)"""
        
        if unit == PowerUnits.MetricHorsepower:
            return f"""{super()._truncate_fraction_digits(self.metric_horsepower, fractional_digits)} hp(M)"""
        
        if unit == PowerUnits.ElectricalHorsepower:
            return f"""{super()._truncate_fraction_digits(self.electrical_horsepower, fractional_digits)} hp(E)"""
        
        if unit == PowerUnits.BoilerHorsepower:
            return f"""{super()._truncate_fraction_digits(self.boiler_horsepower, fractional_digits)} hp(S)"""
        
        if unit == PowerUnits.HydraulicHorsepower:
            return f"""{super()._truncate_fraction_digits(self.hydraulic_horsepower, fractional_digits)} hp(H)"""
        
        if unit == PowerUnits.BritishThermalUnitPerHour:
            return f"""{super()._truncate_fraction_digits(self.british_thermal_units_per_hour, fractional_digits)} Btu/h"""
        
        if unit == PowerUnits.JoulePerHour:
            return f"""{super()._truncate_fraction_digits(self.joules_per_hour, fractional_digits)} J/h"""
        
        if unit == PowerUnits.TonOfRefrigeration:
            return f"""{super()._truncate_fraction_digits(self.tons_of_refrigeration, fractional_digits)} TR"""
        
        if unit == PowerUnits.Femtowatt:
            return f"""{super()._truncate_fraction_digits(self.femtowatts, fractional_digits)} fW"""
        
        if unit == PowerUnits.Picowatt:
            return f"""{super()._truncate_fraction_digits(self.picowatts, fractional_digits)} pW"""
        
        if unit == PowerUnits.Nanowatt:
            return f"""{super()._truncate_fraction_digits(self.nanowatts, fractional_digits)} nW"""
        
        if unit == PowerUnits.Microwatt:
            return f"""{super()._truncate_fraction_digits(self.microwatts, fractional_digits)} μW"""
        
        if unit == PowerUnits.Milliwatt:
            return f"""{super()._truncate_fraction_digits(self.milliwatts, fractional_digits)} mW"""
        
        if unit == PowerUnits.Deciwatt:
            return f"""{super()._truncate_fraction_digits(self.deciwatts, fractional_digits)} dW"""
        
        if unit == PowerUnits.Decawatt:
            return f"""{super()._truncate_fraction_digits(self.decawatts, fractional_digits)} daW"""
        
        if unit == PowerUnits.Kilowatt:
            return f"""{super()._truncate_fraction_digits(self.kilowatts, fractional_digits)} kW"""
        
        if unit == PowerUnits.Megawatt:
            return f"""{super()._truncate_fraction_digits(self.megawatts, fractional_digits)} MW"""
        
        if unit == PowerUnits.Gigawatt:
            return f"""{super()._truncate_fraction_digits(self.gigawatts, fractional_digits)} GW"""
        
        if unit == PowerUnits.Terawatt:
            return f"""{super()._truncate_fraction_digits(self.terawatts, fractional_digits)} TW"""
        
        if unit == PowerUnits.Petawatt:
            return f"""{super()._truncate_fraction_digits(self.petawatts, fractional_digits)} PW"""
        
        if unit == PowerUnits.KilobritishThermalUnitPerHour:
            return f"""{super()._truncate_fraction_digits(self.kilobritish_thermal_units_per_hour, fractional_digits)} kBtu/h"""
        
        if unit == PowerUnits.MegabritishThermalUnitPerHour:
            return f"""{super()._truncate_fraction_digits(self.megabritish_thermal_units_per_hour, fractional_digits)} MBtu/h"""
        
        if unit == PowerUnits.MillijoulePerHour:
            return f"""{super()._truncate_fraction_digits(self.millijoules_per_hour, fractional_digits)} mJ/h"""
        
        if unit == PowerUnits.KilojoulePerHour:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_hour, fractional_digits)} kJ/h"""
        
        if unit == PowerUnits.MegajoulePerHour:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_hour, fractional_digits)} MJ/h"""
        
        if unit == PowerUnits.GigajoulePerHour:
            return f"""{super()._truncate_fraction_digits(self.gigajoules_per_hour, fractional_digits)} GJ/h"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: PowerUnits = PowerUnits.Watt) -> str:
        """
        Get Power unit abbreviation.
        Note! the default abbreviation for Power is Watt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PowerUnits.Watt:
            return """W"""
        
        if unit_abbreviation == PowerUnits.MechanicalHorsepower:
            return """hp(I)"""
        
        if unit_abbreviation == PowerUnits.MetricHorsepower:
            return """hp(M)"""
        
        if unit_abbreviation == PowerUnits.ElectricalHorsepower:
            return """hp(E)"""
        
        if unit_abbreviation == PowerUnits.BoilerHorsepower:
            return """hp(S)"""
        
        if unit_abbreviation == PowerUnits.HydraulicHorsepower:
            return """hp(H)"""
        
        if unit_abbreviation == PowerUnits.BritishThermalUnitPerHour:
            return """Btu/h"""
        
        if unit_abbreviation == PowerUnits.JoulePerHour:
            return """J/h"""
        
        if unit_abbreviation == PowerUnits.TonOfRefrigeration:
            return """TR"""
        
        if unit_abbreviation == PowerUnits.Femtowatt:
            return """fW"""
        
        if unit_abbreviation == PowerUnits.Picowatt:
            return """pW"""
        
        if unit_abbreviation == PowerUnits.Nanowatt:
            return """nW"""
        
        if unit_abbreviation == PowerUnits.Microwatt:
            return """μW"""
        
        if unit_abbreviation == PowerUnits.Milliwatt:
            return """mW"""
        
        if unit_abbreviation == PowerUnits.Deciwatt:
            return """dW"""
        
        if unit_abbreviation == PowerUnits.Decawatt:
            return """daW"""
        
        if unit_abbreviation == PowerUnits.Kilowatt:
            return """kW"""
        
        if unit_abbreviation == PowerUnits.Megawatt:
            return """MW"""
        
        if unit_abbreviation == PowerUnits.Gigawatt:
            return """GW"""
        
        if unit_abbreviation == PowerUnits.Terawatt:
            return """TW"""
        
        if unit_abbreviation == PowerUnits.Petawatt:
            return """PW"""
        
        if unit_abbreviation == PowerUnits.KilobritishThermalUnitPerHour:
            return """kBtu/h"""
        
        if unit_abbreviation == PowerUnits.MegabritishThermalUnitPerHour:
            return """MBtu/h"""
        
        if unit_abbreviation == PowerUnits.MillijoulePerHour:
            return """mJ/h"""
        
        if unit_abbreviation == PowerUnits.KilojoulePerHour:
            return """kJ/h"""
        
        if unit_abbreviation == PowerUnits.MegajoulePerHour:
            return """MJ/h"""
        
        if unit_abbreviation == PowerUnits.GigajoulePerHour:
            return """GJ/h"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MassFractionUnits(Enum):
        """
            MassFractionUnits enumeration
        """
        
        DecimalFraction = 'DecimalFraction'
        """
            
        """
        
        GramPerGram = 'GramPerGram'
        """
            
        """
        
        GramPerKilogram = 'GramPerKilogram'
        """
            
        """
        
        Percent = 'Percent'
        """
            
        """
        
        PartPerThousand = 'PartPerThousand'
        """
            
        """
        
        PartPerMillion = 'PartPerMillion'
        """
            
        """
        
        PartPerBillion = 'PartPerBillion'
        """
            
        """
        
        PartPerTrillion = 'PartPerTrillion'
        """
            
        """
        
        NanogramPerGram = 'NanogramPerGram'
        """
            
        """
        
        MicrogramPerGram = 'MicrogramPerGram'
        """
            
        """
        
        MilligramPerGram = 'MilligramPerGram'
        """
            
        """
        
        CentigramPerGram = 'CentigramPerGram'
        """
            
        """
        
        DecigramPerGram = 'DecigramPerGram'
        """
            
        """
        
        DecagramPerGram = 'DecagramPerGram'
        """
            
        """
        
        HectogramPerGram = 'HectogramPerGram'
        """
            
        """
        
        KilogramPerGram = 'KilogramPerGram'
        """
            
        """
        
        NanogramPerKilogram = 'NanogramPerKilogram'
        """
            
        """
        
        MicrogramPerKilogram = 'MicrogramPerKilogram'
        """
            
        """
        
        MilligramPerKilogram = 'MilligramPerKilogram'
        """
            
        """
        
        CentigramPerKilogram = 'CentigramPerKilogram'
        """
            
        """
        
        DecigramPerKilogram = 'DecigramPerKilogram'
        """
            
        """
        
        DecagramPerKilogram = 'DecagramPerKilogram'
        """
            
        """
        
        HectogramPerKilogram = 'HectogramPerKilogram'
        """
            
        """
        
        KilogramPerKilogram = 'KilogramPerKilogram'
        """
            
        """
        

class MassFractionDto:
    """
    A DTO representation of a MassFraction

    Attributes:
        value (float): The value of the MassFraction.
        unit (MassFractionUnits): The specific unit that the MassFraction value is representing.
    """

    def __init__(self, value: float, unit: MassFractionUnits):
        """
        Create a new DTO representation of a MassFraction

        Parameters:
            value (float): The value of the MassFraction.
            unit (MassFractionUnits): The specific unit that the MassFraction value is representing.
        """
        self.value: float = value
        """
        The value of the MassFraction
        """
        self.unit: MassFractionUnits = unit
        """
        The specific unit that the MassFraction value is representing
        """

    def to_json(self):
        """
        Get a MassFraction DTO JSON object representing the current unit.

        :return: JSON object represents MassFraction DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecimalFraction"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MassFraction DTO from a json representation.

        :param data: The MassFraction DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecimalFraction"}
        :return: A new instance of MassFractionDto.
        :rtype: MassFractionDto
        """
        return MassFractionDto(value=data["value"], unit=MassFractionUnits(data["unit"]))


class MassFraction(AbstractMeasure):
    """
    The mass fraction is defined as the mass of a constituent divided by the total mass of the mixture.

    Args:
        value (float): The value.
        from_unit (MassFractionUnits): The MassFraction unit to create from, The default unit is DecimalFraction
    """
    def __init__(self, value: float, from_unit: MassFractionUnits = MassFractionUnits.DecimalFraction):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__decimal_fractions = None
        
        self.__grams_per_gram = None
        
        self.__grams_per_kilogram = None
        
        self.__percent = None
        
        self.__parts_per_thousand = None
        
        self.__parts_per_million = None
        
        self.__parts_per_billion = None
        
        self.__parts_per_trillion = None
        
        self.__nanograms_per_gram = None
        
        self.__micrograms_per_gram = None
        
        self.__milligrams_per_gram = None
        
        self.__centigrams_per_gram = None
        
        self.__decigrams_per_gram = None
        
        self.__decagrams_per_gram = None
        
        self.__hectograms_per_gram = None
        
        self.__kilograms_per_gram = None
        
        self.__nanograms_per_kilogram = None
        
        self.__micrograms_per_kilogram = None
        
        self.__milligrams_per_kilogram = None
        
        self.__centigrams_per_kilogram = None
        
        self.__decigrams_per_kilogram = None
        
        self.__decagrams_per_kilogram = None
        
        self.__hectograms_per_kilogram = None
        
        self.__kilograms_per_kilogram = None
        

    def convert(self, unit: MassFractionUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MassFractionUnits = MassFractionUnits.DecimalFraction) -> MassFractionDto:
        """
        Get a new instance of MassFraction DTO representing the current unit.

        :param hold_in_unit: The specific MassFraction unit to store the MassFraction value in the DTO representation.
        :type hold_in_unit: MassFractionUnits
        :return: A new instance of MassFractionDto.
        :rtype: MassFractionDto
        """
        return MassFractionDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MassFractionUnits = MassFractionUnits.DecimalFraction):
        """
        Get a MassFraction DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MassFraction unit to store the MassFraction value in the DTO representation.
        :type hold_in_unit: MassFractionUnits
        :return: JSON object represents MassFraction DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecimalFraction"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(mass_fraction_dto: MassFractionDto):
        """
        Obtain a new instance of MassFraction from a DTO unit object.

        :param mass_fraction_dto: The MassFraction DTO representation.
        :type mass_fraction_dto: MassFractionDto
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(mass_fraction_dto.value, mass_fraction_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MassFraction from a DTO unit json representation.

        :param data: The MassFraction DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecimalFraction"}
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction.from_dto(MassFractionDto.from_json(data))

    def __convert_from_base(self, from_unit: MassFractionUnits) -> float:
        value = self._value
        
        if from_unit == MassFractionUnits.DecimalFraction:
            return (value)
        
        if from_unit == MassFractionUnits.GramPerGram:
            return (value)
        
        if from_unit == MassFractionUnits.GramPerKilogram:
            return (value * 1e3)
        
        if from_unit == MassFractionUnits.Percent:
            return (value * 1e2)
        
        if from_unit == MassFractionUnits.PartPerThousand:
            return (value * 1e3)
        
        if from_unit == MassFractionUnits.PartPerMillion:
            return (value * 1e6)
        
        if from_unit == MassFractionUnits.PartPerBillion:
            return (value * 1e9)
        
        if from_unit == MassFractionUnits.PartPerTrillion:
            return (value * 1e12)
        
        if from_unit == MassFractionUnits.NanogramPerGram:
            return ((value) / 1e-09)
        
        if from_unit == MassFractionUnits.MicrogramPerGram:
            return ((value) / 1e-06)
        
        if from_unit == MassFractionUnits.MilligramPerGram:
            return ((value) / 0.001)
        
        if from_unit == MassFractionUnits.CentigramPerGram:
            return ((value) / 0.01)
        
        if from_unit == MassFractionUnits.DecigramPerGram:
            return ((value) / 0.1)
        
        if from_unit == MassFractionUnits.DecagramPerGram:
            return ((value) / 10.0)
        
        if from_unit == MassFractionUnits.HectogramPerGram:
            return ((value) / 100.0)
        
        if from_unit == MassFractionUnits.KilogramPerGram:
            return ((value) / 1000.0)
        
        if from_unit == MassFractionUnits.NanogramPerKilogram:
            return ((value * 1e3) / 1e-09)
        
        if from_unit == MassFractionUnits.MicrogramPerKilogram:
            return ((value * 1e3) / 1e-06)
        
        if from_unit == MassFractionUnits.MilligramPerKilogram:
            return ((value * 1e3) / 0.001)
        
        if from_unit == MassFractionUnits.CentigramPerKilogram:
            return ((value * 1e3) / 0.01)
        
        if from_unit == MassFractionUnits.DecigramPerKilogram:
            return ((value * 1e3) / 0.1)
        
        if from_unit == MassFractionUnits.DecagramPerKilogram:
            return ((value * 1e3) / 10.0)
        
        if from_unit == MassFractionUnits.HectogramPerKilogram:
            return ((value * 1e3) / 100.0)
        
        if from_unit == MassFractionUnits.KilogramPerKilogram:
            return ((value * 1e3) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MassFractionUnits) -> float:
        
        if to_unit == MassFractionUnits.DecimalFraction:
            return (value)
        
        if to_unit == MassFractionUnits.GramPerGram:
            return (value)
        
        if to_unit == MassFractionUnits.GramPerKilogram:
            return (value / 1e3)
        
        if to_unit == MassFractionUnits.Percent:
            return (value / 1e2)
        
        if to_unit == MassFractionUnits.PartPerThousand:
            return (value / 1e3)
        
        if to_unit == MassFractionUnits.PartPerMillion:
            return (value / 1e6)
        
        if to_unit == MassFractionUnits.PartPerBillion:
            return (value / 1e9)
        
        if to_unit == MassFractionUnits.PartPerTrillion:
            return (value / 1e12)
        
        if to_unit == MassFractionUnits.NanogramPerGram:
            return ((value) * 1e-09)
        
        if to_unit == MassFractionUnits.MicrogramPerGram:
            return ((value) * 1e-06)
        
        if to_unit == MassFractionUnits.MilligramPerGram:
            return ((value) * 0.001)
        
        if to_unit == MassFractionUnits.CentigramPerGram:
            return ((value) * 0.01)
        
        if to_unit == MassFractionUnits.DecigramPerGram:
            return ((value) * 0.1)
        
        if to_unit == MassFractionUnits.DecagramPerGram:
            return ((value) * 10.0)
        
        if to_unit == MassFractionUnits.HectogramPerGram:
            return ((value) * 100.0)
        
        if to_unit == MassFractionUnits.KilogramPerGram:
            return ((value) * 1000.0)
        
        if to_unit == MassFractionUnits.NanogramPerKilogram:
            return ((value / 1e3) * 1e-09)
        
        if to_unit == MassFractionUnits.MicrogramPerKilogram:
            return ((value / 1e3) * 1e-06)
        
        if to_unit == MassFractionUnits.MilligramPerKilogram:
            return ((value / 1e3) * 0.001)
        
        if to_unit == MassFractionUnits.CentigramPerKilogram:
            return ((value / 1e3) * 0.01)
        
        if to_unit == MassFractionUnits.DecigramPerKilogram:
            return ((value / 1e3) * 0.1)
        
        if to_unit == MassFractionUnits.DecagramPerKilogram:
            return ((value / 1e3) * 10.0)
        
        if to_unit == MassFractionUnits.HectogramPerKilogram:
            return ((value / 1e3) * 100.0)
        
        if to_unit == MassFractionUnits.KilogramPerKilogram:
            return ((value / 1e3) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_decimal_fractions(decimal_fractions: float):
        """
        Create a new instance of MassFraction from a value in decimal_fractions.

        

        :param meters: The MassFraction value in decimal_fractions.
        :type decimal_fractions: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(decimal_fractions, MassFractionUnits.DecimalFraction)

    
    @staticmethod
    def from_grams_per_gram(grams_per_gram: float):
        """
        Create a new instance of MassFraction from a value in grams_per_gram.

        

        :param meters: The MassFraction value in grams_per_gram.
        :type grams_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(grams_per_gram, MassFractionUnits.GramPerGram)

    
    @staticmethod
    def from_grams_per_kilogram(grams_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in grams_per_kilogram.

        

        :param meters: The MassFraction value in grams_per_kilogram.
        :type grams_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(grams_per_kilogram, MassFractionUnits.GramPerKilogram)

    
    @staticmethod
    def from_percent(percent: float):
        """
        Create a new instance of MassFraction from a value in percent.

        

        :param meters: The MassFraction value in percent.
        :type percent: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(percent, MassFractionUnits.Percent)

    
    @staticmethod
    def from_parts_per_thousand(parts_per_thousand: float):
        """
        Create a new instance of MassFraction from a value in parts_per_thousand.

        

        :param meters: The MassFraction value in parts_per_thousand.
        :type parts_per_thousand: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(parts_per_thousand, MassFractionUnits.PartPerThousand)

    
    @staticmethod
    def from_parts_per_million(parts_per_million: float):
        """
        Create a new instance of MassFraction from a value in parts_per_million.

        

        :param meters: The MassFraction value in parts_per_million.
        :type parts_per_million: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(parts_per_million, MassFractionUnits.PartPerMillion)

    
    @staticmethod
    def from_parts_per_billion(parts_per_billion: float):
        """
        Create a new instance of MassFraction from a value in parts_per_billion.

        

        :param meters: The MassFraction value in parts_per_billion.
        :type parts_per_billion: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(parts_per_billion, MassFractionUnits.PartPerBillion)

    
    @staticmethod
    def from_parts_per_trillion(parts_per_trillion: float):
        """
        Create a new instance of MassFraction from a value in parts_per_trillion.

        

        :param meters: The MassFraction value in parts_per_trillion.
        :type parts_per_trillion: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(parts_per_trillion, MassFractionUnits.PartPerTrillion)

    
    @staticmethod
    def from_nanograms_per_gram(nanograms_per_gram: float):
        """
        Create a new instance of MassFraction from a value in nanograms_per_gram.

        

        :param meters: The MassFraction value in nanograms_per_gram.
        :type nanograms_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(nanograms_per_gram, MassFractionUnits.NanogramPerGram)

    
    @staticmethod
    def from_micrograms_per_gram(micrograms_per_gram: float):
        """
        Create a new instance of MassFraction from a value in micrograms_per_gram.

        

        :param meters: The MassFraction value in micrograms_per_gram.
        :type micrograms_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(micrograms_per_gram, MassFractionUnits.MicrogramPerGram)

    
    @staticmethod
    def from_milligrams_per_gram(milligrams_per_gram: float):
        """
        Create a new instance of MassFraction from a value in milligrams_per_gram.

        

        :param meters: The MassFraction value in milligrams_per_gram.
        :type milligrams_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(milligrams_per_gram, MassFractionUnits.MilligramPerGram)

    
    @staticmethod
    def from_centigrams_per_gram(centigrams_per_gram: float):
        """
        Create a new instance of MassFraction from a value in centigrams_per_gram.

        

        :param meters: The MassFraction value in centigrams_per_gram.
        :type centigrams_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(centigrams_per_gram, MassFractionUnits.CentigramPerGram)

    
    @staticmethod
    def from_decigrams_per_gram(decigrams_per_gram: float):
        """
        Create a new instance of MassFraction from a value in decigrams_per_gram.

        

        :param meters: The MassFraction value in decigrams_per_gram.
        :type decigrams_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(decigrams_per_gram, MassFractionUnits.DecigramPerGram)

    
    @staticmethod
    def from_decagrams_per_gram(decagrams_per_gram: float):
        """
        Create a new instance of MassFraction from a value in decagrams_per_gram.

        

        :param meters: The MassFraction value in decagrams_per_gram.
        :type decagrams_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(decagrams_per_gram, MassFractionUnits.DecagramPerGram)

    
    @staticmethod
    def from_hectograms_per_gram(hectograms_per_gram: float):
        """
        Create a new instance of MassFraction from a value in hectograms_per_gram.

        

        :param meters: The MassFraction value in hectograms_per_gram.
        :type hectograms_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(hectograms_per_gram, MassFractionUnits.HectogramPerGram)

    
    @staticmethod
    def from_kilograms_per_gram(kilograms_per_gram: float):
        """
        Create a new instance of MassFraction from a value in kilograms_per_gram.

        

        :param meters: The MassFraction value in kilograms_per_gram.
        :type kilograms_per_gram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(kilograms_per_gram, MassFractionUnits.KilogramPerGram)

    
    @staticmethod
    def from_nanograms_per_kilogram(nanograms_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in nanograms_per_kilogram.

        

        :param meters: The MassFraction value in nanograms_per_kilogram.
        :type nanograms_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(nanograms_per_kilogram, MassFractionUnits.NanogramPerKilogram)

    
    @staticmethod
    def from_micrograms_per_kilogram(micrograms_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in micrograms_per_kilogram.

        

        :param meters: The MassFraction value in micrograms_per_kilogram.
        :type micrograms_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(micrograms_per_kilogram, MassFractionUnits.MicrogramPerKilogram)

    
    @staticmethod
    def from_milligrams_per_kilogram(milligrams_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in milligrams_per_kilogram.

        

        :param meters: The MassFraction value in milligrams_per_kilogram.
        :type milligrams_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(milligrams_per_kilogram, MassFractionUnits.MilligramPerKilogram)

    
    @staticmethod
    def from_centigrams_per_kilogram(centigrams_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in centigrams_per_kilogram.

        

        :param meters: The MassFraction value in centigrams_per_kilogram.
        :type centigrams_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(centigrams_per_kilogram, MassFractionUnits.CentigramPerKilogram)

    
    @staticmethod
    def from_decigrams_per_kilogram(decigrams_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in decigrams_per_kilogram.

        

        :param meters: The MassFraction value in decigrams_per_kilogram.
        :type decigrams_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(decigrams_per_kilogram, MassFractionUnits.DecigramPerKilogram)

    
    @staticmethod
    def from_decagrams_per_kilogram(decagrams_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in decagrams_per_kilogram.

        

        :param meters: The MassFraction value in decagrams_per_kilogram.
        :type decagrams_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(decagrams_per_kilogram, MassFractionUnits.DecagramPerKilogram)

    
    @staticmethod
    def from_hectograms_per_kilogram(hectograms_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in hectograms_per_kilogram.

        

        :param meters: The MassFraction value in hectograms_per_kilogram.
        :type hectograms_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(hectograms_per_kilogram, MassFractionUnits.HectogramPerKilogram)

    
    @staticmethod
    def from_kilograms_per_kilogram(kilograms_per_kilogram: float):
        """
        Create a new instance of MassFraction from a value in kilograms_per_kilogram.

        

        :param meters: The MassFraction value in kilograms_per_kilogram.
        :type kilograms_per_kilogram: float
        :return: A new instance of MassFraction.
        :rtype: MassFraction
        """
        return MassFraction(kilograms_per_kilogram, MassFractionUnits.KilogramPerKilogram)

    
    @property
    def decimal_fractions(self) -> float:
        """
        
        """
        if self.__decimal_fractions != None:
            return self.__decimal_fractions
        self.__decimal_fractions = self.__convert_from_base(MassFractionUnits.DecimalFraction)
        return self.__decimal_fractions

    
    @property
    def grams_per_gram(self) -> float:
        """
        
        """
        if self.__grams_per_gram != None:
            return self.__grams_per_gram
        self.__grams_per_gram = self.__convert_from_base(MassFractionUnits.GramPerGram)
        return self.__grams_per_gram

    
    @property
    def grams_per_kilogram(self) -> float:
        """
        
        """
        if self.__grams_per_kilogram != None:
            return self.__grams_per_kilogram
        self.__grams_per_kilogram = self.__convert_from_base(MassFractionUnits.GramPerKilogram)
        return self.__grams_per_kilogram

    
    @property
    def percent(self) -> float:
        """
        
        """
        if self.__percent != None:
            return self.__percent
        self.__percent = self.__convert_from_base(MassFractionUnits.Percent)
        return self.__percent

    
    @property
    def parts_per_thousand(self) -> float:
        """
        
        """
        if self.__parts_per_thousand != None:
            return self.__parts_per_thousand
        self.__parts_per_thousand = self.__convert_from_base(MassFractionUnits.PartPerThousand)
        return self.__parts_per_thousand

    
    @property
    def parts_per_million(self) -> float:
        """
        
        """
        if self.__parts_per_million != None:
            return self.__parts_per_million
        self.__parts_per_million = self.__convert_from_base(MassFractionUnits.PartPerMillion)
        return self.__parts_per_million

    
    @property
    def parts_per_billion(self) -> float:
        """
        
        """
        if self.__parts_per_billion != None:
            return self.__parts_per_billion
        self.__parts_per_billion = self.__convert_from_base(MassFractionUnits.PartPerBillion)
        return self.__parts_per_billion

    
    @property
    def parts_per_trillion(self) -> float:
        """
        
        """
        if self.__parts_per_trillion != None:
            return self.__parts_per_trillion
        self.__parts_per_trillion = self.__convert_from_base(MassFractionUnits.PartPerTrillion)
        return self.__parts_per_trillion

    
    @property
    def nanograms_per_gram(self) -> float:
        """
        
        """
        if self.__nanograms_per_gram != None:
            return self.__nanograms_per_gram
        self.__nanograms_per_gram = self.__convert_from_base(MassFractionUnits.NanogramPerGram)
        return self.__nanograms_per_gram

    
    @property
    def micrograms_per_gram(self) -> float:
        """
        
        """
        if self.__micrograms_per_gram != None:
            return self.__micrograms_per_gram
        self.__micrograms_per_gram = self.__convert_from_base(MassFractionUnits.MicrogramPerGram)
        return self.__micrograms_per_gram

    
    @property
    def milligrams_per_gram(self) -> float:
        """
        
        """
        if self.__milligrams_per_gram != None:
            return self.__milligrams_per_gram
        self.__milligrams_per_gram = self.__convert_from_base(MassFractionUnits.MilligramPerGram)
        return self.__milligrams_per_gram

    
    @property
    def centigrams_per_gram(self) -> float:
        """
        
        """
        if self.__centigrams_per_gram != None:
            return self.__centigrams_per_gram
        self.__centigrams_per_gram = self.__convert_from_base(MassFractionUnits.CentigramPerGram)
        return self.__centigrams_per_gram

    
    @property
    def decigrams_per_gram(self) -> float:
        """
        
        """
        if self.__decigrams_per_gram != None:
            return self.__decigrams_per_gram
        self.__decigrams_per_gram = self.__convert_from_base(MassFractionUnits.DecigramPerGram)
        return self.__decigrams_per_gram

    
    @property
    def decagrams_per_gram(self) -> float:
        """
        
        """
        if self.__decagrams_per_gram != None:
            return self.__decagrams_per_gram
        self.__decagrams_per_gram = self.__convert_from_base(MassFractionUnits.DecagramPerGram)
        return self.__decagrams_per_gram

    
    @property
    def hectograms_per_gram(self) -> float:
        """
        
        """
        if self.__hectograms_per_gram != None:
            return self.__hectograms_per_gram
        self.__hectograms_per_gram = self.__convert_from_base(MassFractionUnits.HectogramPerGram)
        return self.__hectograms_per_gram

    
    @property
    def kilograms_per_gram(self) -> float:
        """
        
        """
        if self.__kilograms_per_gram != None:
            return self.__kilograms_per_gram
        self.__kilograms_per_gram = self.__convert_from_base(MassFractionUnits.KilogramPerGram)
        return self.__kilograms_per_gram

    
    @property
    def nanograms_per_kilogram(self) -> float:
        """
        
        """
        if self.__nanograms_per_kilogram != None:
            return self.__nanograms_per_kilogram
        self.__nanograms_per_kilogram = self.__convert_from_base(MassFractionUnits.NanogramPerKilogram)
        return self.__nanograms_per_kilogram

    
    @property
    def micrograms_per_kilogram(self) -> float:
        """
        
        """
        if self.__micrograms_per_kilogram != None:
            return self.__micrograms_per_kilogram
        self.__micrograms_per_kilogram = self.__convert_from_base(MassFractionUnits.MicrogramPerKilogram)
        return self.__micrograms_per_kilogram

    
    @property
    def milligrams_per_kilogram(self) -> float:
        """
        
        """
        if self.__milligrams_per_kilogram != None:
            return self.__milligrams_per_kilogram
        self.__milligrams_per_kilogram = self.__convert_from_base(MassFractionUnits.MilligramPerKilogram)
        return self.__milligrams_per_kilogram

    
    @property
    def centigrams_per_kilogram(self) -> float:
        """
        
        """
        if self.__centigrams_per_kilogram != None:
            return self.__centigrams_per_kilogram
        self.__centigrams_per_kilogram = self.__convert_from_base(MassFractionUnits.CentigramPerKilogram)
        return self.__centigrams_per_kilogram

    
    @property
    def decigrams_per_kilogram(self) -> float:
        """
        
        """
        if self.__decigrams_per_kilogram != None:
            return self.__decigrams_per_kilogram
        self.__decigrams_per_kilogram = self.__convert_from_base(MassFractionUnits.DecigramPerKilogram)
        return self.__decigrams_per_kilogram

    
    @property
    def decagrams_per_kilogram(self) -> float:
        """
        
        """
        if self.__decagrams_per_kilogram != None:
            return self.__decagrams_per_kilogram
        self.__decagrams_per_kilogram = self.__convert_from_base(MassFractionUnits.DecagramPerKilogram)
        return self.__decagrams_per_kilogram

    
    @property
    def hectograms_per_kilogram(self) -> float:
        """
        
        """
        if self.__hectograms_per_kilogram != None:
            return self.__hectograms_per_kilogram
        self.__hectograms_per_kilogram = self.__convert_from_base(MassFractionUnits.HectogramPerKilogram)
        return self.__hectograms_per_kilogram

    
    @property
    def kilograms_per_kilogram(self) -> float:
        """
        
        """
        if self.__kilograms_per_kilogram != None:
            return self.__kilograms_per_kilogram
        self.__kilograms_per_kilogram = self.__convert_from_base(MassFractionUnits.KilogramPerKilogram)
        return self.__kilograms_per_kilogram

    
    def to_string(self, unit: MassFractionUnits = MassFractionUnits.DecimalFraction, fractional_digits: int = None) -> str:
        """
        Format the MassFraction to a string.
        
        Note: the default format for MassFraction is DecimalFraction.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MassFraction. Default is 'DecimalFraction'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MassFractionUnits.DecimalFraction:
            return f"""{super()._truncate_fraction_digits(self.decimal_fractions, fractional_digits)} """
        
        if unit == MassFractionUnits.GramPerGram:
            return f"""{super()._truncate_fraction_digits(self.grams_per_gram, fractional_digits)} g/g"""
        
        if unit == MassFractionUnits.GramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.grams_per_kilogram, fractional_digits)} g/kg"""
        
        if unit == MassFractionUnits.Percent:
            return f"""{super()._truncate_fraction_digits(self.percent, fractional_digits)} %"""
        
        if unit == MassFractionUnits.PartPerThousand:
            return f"""{super()._truncate_fraction_digits(self.parts_per_thousand, fractional_digits)} ‰"""
        
        if unit == MassFractionUnits.PartPerMillion:
            return f"""{super()._truncate_fraction_digits(self.parts_per_million, fractional_digits)} ppm"""
        
        if unit == MassFractionUnits.PartPerBillion:
            return f"""{super()._truncate_fraction_digits(self.parts_per_billion, fractional_digits)} ppb"""
        
        if unit == MassFractionUnits.PartPerTrillion:
            return f"""{super()._truncate_fraction_digits(self.parts_per_trillion, fractional_digits)} ppt"""
        
        if unit == MassFractionUnits.NanogramPerGram:
            return f"""{super()._truncate_fraction_digits(self.nanograms_per_gram, fractional_digits)} ng/g"""
        
        if unit == MassFractionUnits.MicrogramPerGram:
            return f"""{super()._truncate_fraction_digits(self.micrograms_per_gram, fractional_digits)} μg/g"""
        
        if unit == MassFractionUnits.MilligramPerGram:
            return f"""{super()._truncate_fraction_digits(self.milligrams_per_gram, fractional_digits)} mg/g"""
        
        if unit == MassFractionUnits.CentigramPerGram:
            return f"""{super()._truncate_fraction_digits(self.centigrams_per_gram, fractional_digits)} cg/g"""
        
        if unit == MassFractionUnits.DecigramPerGram:
            return f"""{super()._truncate_fraction_digits(self.decigrams_per_gram, fractional_digits)} dg/g"""
        
        if unit == MassFractionUnits.DecagramPerGram:
            return f"""{super()._truncate_fraction_digits(self.decagrams_per_gram, fractional_digits)} dag/g"""
        
        if unit == MassFractionUnits.HectogramPerGram:
            return f"""{super()._truncate_fraction_digits(self.hectograms_per_gram, fractional_digits)} hg/g"""
        
        if unit == MassFractionUnits.KilogramPerGram:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_gram, fractional_digits)} kg/g"""
        
        if unit == MassFractionUnits.NanogramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.nanograms_per_kilogram, fractional_digits)} ng/kg"""
        
        if unit == MassFractionUnits.MicrogramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.micrograms_per_kilogram, fractional_digits)} μg/kg"""
        
        if unit == MassFractionUnits.MilligramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.milligrams_per_kilogram, fractional_digits)} mg/kg"""
        
        if unit == MassFractionUnits.CentigramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.centigrams_per_kilogram, fractional_digits)} cg/kg"""
        
        if unit == MassFractionUnits.DecigramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.decigrams_per_kilogram, fractional_digits)} dg/kg"""
        
        if unit == MassFractionUnits.DecagramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.decagrams_per_kilogram, fractional_digits)} dag/kg"""
        
        if unit == MassFractionUnits.HectogramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.hectograms_per_kilogram, fractional_digits)} hg/kg"""
        
        if unit == MassFractionUnits.KilogramPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_kilogram, fractional_digits)} kg/kg"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MassFractionUnits = MassFractionUnits.DecimalFraction) -> str:
        """
        Get MassFraction unit abbreviation.
        Note! the default abbreviation for MassFraction is DecimalFraction.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MassFractionUnits.DecimalFraction:
            return """"""
        
        if unit_abbreviation == MassFractionUnits.GramPerGram:
            return """g/g"""
        
        if unit_abbreviation == MassFractionUnits.GramPerKilogram:
            return """g/kg"""
        
        if unit_abbreviation == MassFractionUnits.Percent:
            return """%"""
        
        if unit_abbreviation == MassFractionUnits.PartPerThousand:
            return """‰"""
        
        if unit_abbreviation == MassFractionUnits.PartPerMillion:
            return """ppm"""
        
        if unit_abbreviation == MassFractionUnits.PartPerBillion:
            return """ppb"""
        
        if unit_abbreviation == MassFractionUnits.PartPerTrillion:
            return """ppt"""
        
        if unit_abbreviation == MassFractionUnits.NanogramPerGram:
            return """ng/g"""
        
        if unit_abbreviation == MassFractionUnits.MicrogramPerGram:
            return """μg/g"""
        
        if unit_abbreviation == MassFractionUnits.MilligramPerGram:
            return """mg/g"""
        
        if unit_abbreviation == MassFractionUnits.CentigramPerGram:
            return """cg/g"""
        
        if unit_abbreviation == MassFractionUnits.DecigramPerGram:
            return """dg/g"""
        
        if unit_abbreviation == MassFractionUnits.DecagramPerGram:
            return """dag/g"""
        
        if unit_abbreviation == MassFractionUnits.HectogramPerGram:
            return """hg/g"""
        
        if unit_abbreviation == MassFractionUnits.KilogramPerGram:
            return """kg/g"""
        
        if unit_abbreviation == MassFractionUnits.NanogramPerKilogram:
            return """ng/kg"""
        
        if unit_abbreviation == MassFractionUnits.MicrogramPerKilogram:
            return """μg/kg"""
        
        if unit_abbreviation == MassFractionUnits.MilligramPerKilogram:
            return """mg/kg"""
        
        if unit_abbreviation == MassFractionUnits.CentigramPerKilogram:
            return """cg/kg"""
        
        if unit_abbreviation == MassFractionUnits.DecigramPerKilogram:
            return """dg/kg"""
        
        if unit_abbreviation == MassFractionUnits.DecagramPerKilogram:
            return """dag/kg"""
        
        if unit_abbreviation == MassFractionUnits.HectogramPerKilogram:
            return """hg/kg"""
        
        if unit_abbreviation == MassFractionUnits.KilogramPerKilogram:
            return """kg/kg"""
        